/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */

package org.gvsig.fmap.dal.store.jdbc;

import org.gvsig.fmap.dal.DataStoreParameters;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.BaseTestEditableFeatureStore;
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.EditableFeatureType;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureType;
import org.gvsig.fmap.dal.feature.NewFeatureStoreParameters;

import com.mysql.jdbc.Driver;


public class JDBCTestWrite extends BaseTestEditableFeatureStore {
	private JDBCNewStoreParameters newParams;
	private JDBCServerExplorer myExplorer;
	private int newId = 50000;

	public void fillPrimaryKeyInserFeature(EditableFeature feature) {
		feature.set("id", newId);
		newId++;
	}

	@Override
	protected void doSetUp() throws Exception {
		// Nothing to do	
	}

	public DataStoreParameters getDefaultDataStoreParameters()
			throws DataException {
		JDBCStoreParameters parameters = null;
		parameters = (JDBCStoreParameters) dataManager
				.createStoreParameters(JDBCStoreProvider.NAME);

		parameters.setHost("localhost");
		parameters.setPort(3306);
		parameters.setUser("test");
		parameters.setPassword("test");
		parameters.setDBName("gis");
		parameters.setTable("alfanum");
		parameters.setUrl("jdbc:mysql://" + parameters.getHost() + ":"
				+ parameters.getPort() + "/" + parameters.getDBName());
		parameters.setJDBCDriverClassName(Driver.class.getName());

		return parameters;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.gvsig.fmap.dal.feature.BaseTestFeatureStore#hasExplorer()
	 */
	@Override
	public boolean hasExplorer() {
		// TODO Auto-generated method stub
		return true;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.gvsig.fmap.dal.feature.BaseTestFeatureStore#usesResources()
	 */
	@Override
	public boolean usesResources() {
		// TODO Auto-generated method stub
		return true;
	}

	/*
	public void testImportDB() throws Exception {
		JDBCNewStoreParameters newParasm = (JDBCNewStoreParameters) getDefaultNewDataStoreParameters();
		JDBCServerExplorerParameters seParams = (JDBCServerExplorerParameters) dataManager
				.createServerExplorerParameters(JDBCServerExplorer.NAME);

		assertTrue(myExplorer.add(newParasm, true));

		FeatureStore store = (FeatureStore) dataManager
				.createStore(getDefaultDataStoreParameters());

		FeatureStore newstore = (FeatureStore) dataManager
				.createStore(newParasm);



		newstore.edit(FeatureStore.MODE_APPEND);
		FeatureSet set = store.getFeatureSet();
		DisposableIterator iter = set.iterator();
		Feature org;
		EditableFeature trg;
		Iterator attrIter;
		FeatureAttributeDescriptor attr;
		while (iter.hasNext()) {
			org = (Feature) iter.next();
			trg = newstore.createNewFeature(false);
			attrIter = trg.getType().iterator();
			while (attrIter.hasNext()) {
				attr = (FeatureAttributeDescriptor) attrIter.next();
				trg.set(attr.getName(), org.get(attr.getName()));
			}
			newstore.insert(trg);
		}
		newstore.finishEditing();

		iter.dispose();
		set.dispose();
		store.dispose();
		newstore.dispose();


	}
	*/

	@Override
	public NewFeatureStoreParameters getDefaultNewDataStoreParameters()
			throws Exception {


		JDBCStoreParameters params = (JDBCStoreParameters) getDefaultDataStoreParameters();

		if (this.myExplorer == null) {

			JDBCServerExplorerParameters seParameters = null;
			seParameters = (JDBCServerExplorerParameters) dataManager
					.createServerExplorerParameters(JDBCServerExplorer.NAME);

			seParameters.setHost("localhost");
			seParameters.setPort(3306);
			seParameters.setUser("test");
			seParameters.setPassword("test");
			seParameters.setDBName("gis");

			seParameters.setUrl("jdbc:mysql://" + seParameters.getHost() + ":"
					+ seParameters.getPort() + "/" + seParameters.getDBName());
			seParameters.setJDBCDriverClassName(Driver.class.getName());


			myExplorer = (JDBCServerExplorer) dataManager
					.createServerExplorer(seParameters);
		}
		if (this.newParams == null) {
			FeatureStore store = (FeatureStore) dataManager.createStore(params);

			newParams = (JDBCNewStoreParameters) myExplorer
					.getAddParameters();


//			newParams.setTable(params.getTable() + "_test");
			FeatureType ftOrg = store.getDefaultFeatureType();
			EditableFeatureType ftTrg = ftOrg.getEditable();

//			EditableFeatureType ftTrg = (EditableFeatureType) newParams
//					.getDefaultFeatureType();
			newParams.setDefaultFeatureType(ftTrg);

//			FeatureAttributeDescriptor org;
//			EditableFeatureAttributeDescriptor trg;
//			Iterator iter = ftOrg.iterator();
//			while (iter.hasNext()) {
//				org = (FeatureAttributeDescriptor) iter.next();
//				trg = ftTrg.add(org.getName(), org.getDataType());
//				trg.setAllowNull(org.allowNull());
//				trg.setDefaultValue(org.getDefaultValue());
//				trg.setGeometrySubType(org.getGeometrySubType());
//				trg.setGeometryType(org.getGeometryType());
//				trg.setIsAutomatic(org.isAutomatic());
//				trg.setIsPrimaryKey(org.isPrimaryKey());
//				trg.setIsReadOnly(org.isReadOnly());
//				trg.setMaximumOccurrences(org.getMaximumOccurrences());
//				trg.setMinimumOccurrences(org.getMinimumOccurrences());
//				trg.setPrecision(org.getPrecision());
//				trg.setSize(org.getSize());
//				trg.setSRS(org.getSRS());
//				trg.setAdditionalInfo("SQLType", org
//						.getAdditionalInfo("SQLType"));
//				trg.setAdditionalInfo("SQLTypeName", org
//						.getAdditionalInfo("SQLTypeName"));
//
//			}
//			ftTrg.setDefaultGeometryAttributeName(ftOrg
//					.getDefaultGeometryAttributeName());
//			ftTrg.setHasOID(ftOrg.hasOID());

			store.dispose();
		}

		return this.newParams;
	}

	@Override
	public boolean resourcesNotifyChanges() {
		// TODO Auto-generated method stub
		return false;
	}

}
