/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.feature.impl.expansionadapter;

import java.util.HashMap;
import java.util.Iterator;


/**
 * Maneja el fichero de extensin en el que se almacenan las modificacionesy adicines
 * durante la edicin. Los ndices que devuelve esta clase en sus mtodos
 * addFeature y modifyFeature son invariables, es decir, que si se invoca un
 * mtodo addFeature que retorna un 8, independientemente de las operaciones
 * que se realicen posteriormente, una llamada a getFeature(8) retornar
 * dicha fila. Si esta geometra es eliminada posteriormente, se retornar un
 * null. Esto ltimo se cumple mientras no se invoque el mtodo compact, mediante
 * el que se reorganizan las geometras dejando en el fichero slo las que tienen
 * validez en el momento de realizar la invocacin.
 *
 * @author Vicente Caballero Navarro
 *
 */
public interface ExpansionAdapter {
	/**
	 * Aade una geometria al final del fichero y retorna el ndice que ocupa
	 * esta geometria en el mismo
	 *
	 * @param feature
	 * @param status TODO
	 * @param indexInternalFields fields that where valid when this feature was added.
	 *
	 * @return calculated index of the new feature.
	 */
	int addObject(Object obj);

	/**
	 * Modifica la index-sima geometra del fichero devolviendo la posicin en
	 * la que se pone la geometria modificada.
	 *
	 * @param calculated index of feature to be modified
	 * @param feature newFeature that replaces the old feature.
	 *
	 * @return new calculated index of the modified feature.
	 */
//	int updateObject(Object obj);

	/**
	 * Obtiene la geometria que hay en el ndice 'index' o null si la geometra
	 * ha sido invalidada.
	 *
	 * @param index caculatedIndex of the feature to be read.
	 * @return feature
	 */
	Object getObject(int index);

	/**
	 * Realiza una compactacin del fichero que maneja esta clase
	 *
	 * @param relations DOCUMENT ME!
	 */
	void compact(HashMap relations);

	/**
     * Mueve el puntero de escritura de manera que las siguientes escrituras
     * machacarn la ltima fila
     */
    void deleteLastObject();

    /**
     * Abre el fichero de expansin para comenzar la edicin
     */
    void open();

    /**
     * Cierra el fichero de expansin al terminar la edicin
     */
    void close();

    int getSize();

    Iterator iterator();
}
