/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.fmap.dal.feature.impl.featureset;

import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.dynobject.exception.DynFieldNotFoundException;
import org.gvsig.tools.dynobject.exception.DynMethodException;
import org.gvsig.tools.lang.Cloneable;

/**
 * {@link DynObject} implementation to facade a Feature and allow to be used
 * as a {@link DynObject}.
 * 
 * This implementation may be reused to be used with many {@link Feature}
 * objects, but not at the same time.
 * 
 * @author gvSIG Team
 * @version $Id$
 */
public class DynObjectFeatureFacade implements DynObject , Cloneable{

    private Feature feature;
    
    private EditableFeature editable;

    private final Object lock = new Object();

    /**
     * Empty constructor.
     */
    public DynObjectFeatureFacade() {
        // Nothing to do
    }

    /**
     * Creates a facade over a {@link Feature}.
     */
    public DynObjectFeatureFacade(Feature feature) {
        setFeatureCopy(feature);
    }

    public DynClass getDynClass() {
        return getFeature().getType();
    }

    public Object getDynValue(String name) throws DynFieldNotFoundException {
        return getFeature().get(name);
    }

    public void setDynValue(String name, Object value)
        throws DynFieldNotFoundException {
        synchronized (lock) {
            if (editable == null) {
                editable = feature.getEditable();
            }            
        }
        editable.set(name, value);
    }

    public boolean hasDynValue(String name) {
        return getFeature().get(name) != null;
    }

    public Object invokeDynMethod(String name, DynObject context)
        throws DynMethodException {
        throw new UnsupportedOperationException();
    }

    public Object invokeDynMethod(int code, DynObject context)
        throws DynMethodException {
        throw new UnsupportedOperationException();
    }

    public void implement(DynClass dynClass) {
        throw new UnsupportedOperationException();
    }

    public void delegate(DynObject dynObject) {
        throw new UnsupportedOperationException();
    }

    public void clear() {
        // Nothing to do
    }
    
    public Feature getFeature() {
        return editable == null ? feature : editable;
    }
    
    public void setFeatureCopy(Feature feature) {
        synchronized (lock) {
            this.feature = feature.getCopy();
            this.editable = null;
        }
    }

    public void setFeature(Feature feature) {
        synchronized (lock) {
            this.feature = feature;
            this.editable = null;
        }
    }

    public EditableFeature getEditable() {
        return editable;
    }
    
    public String toString() {
	    if (editable != null) {
	        return editable.toString();
	    }            
	    return this.feature.toString();
    }

    public Object clone() throws CloneNotSupportedException {
    	DynObjectFeatureFacade other = new DynObjectFeatureFacade();
    	if( feature != null ) {
    		other.feature = this.feature.getCopy();
    	}
    	if( editable != null ) {
    		other.editable = (EditableFeature) this.editable.getCopy();
    	}
    	return other;
    }
}
