/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */

package org.gvsig.fmap.dal.spi;

//import com.google.gson.stream.JsonReader;
//import com.google.gson.stream.JsonToken;
import java.io.StringWriter;
import java.util.Iterator;

import org.gvsig.fmap.dal.DataParameters;
import org.gvsig.fmap.dal.exception.CopyParametersException;
import org.gvsig.fmap.dal.exception.ValidateDataParametersException;
import org.gvsig.tools.dynobject.DelegatedDynObject;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynField;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.dynobject.exception.DynMethodException;
import org.gvsig.tools.dynobject.exception.DynObjectValidateException;
import org.gvsig.tools.persistence.PersistentState;
import org.gvsig.tools.persistence.exception.PersistenceException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
//import com.google.gson.stream.JsonWriter;
//import java.io.IOException;
//import java.io.StringReader;

/**
 * @author jmvivo
 *
 */
public abstract class AbstractDataParameters implements DataParameters {
	private static final Logger logger = LoggerFactory.getLogger(AbstractDataParameters.class);

	public Object getDynValue(String name) {
		return getDelegatedDynObject().getDynValue(name);
	}

	public String toString() {
		return getDelegatedDynObject().toString();
	}

//        public boolean fromString(String values) {
//            boolean ok = true;
//            try {
//                DynObject obj = getDelegatedDynObject();
//                JsonReader reader = new JsonReader(new StringReader(values));
//                reader.setLenient(true);
//                reader.beginObject();
//                String name = null;
//                String value = null;
//                while (reader.hasNext()) {
//                    name ="unknow";
//                    try {
//                        name = reader.nextName();
//                        if (reader.peek() == JsonToken.NULL) {
//                            value = null;
//                        } else {
//                            value = reader.nextString();
//                        }
//                        obj.setDynValue(name, value);
//                    } catch (Exception ex) {
//                        logger.warn("Can't restore parameter '"+name+"' from string "+values+".", ex);
//                        ok = false;
//                    }
//                }
//                reader.endObject();
//            } catch (IOException ex) {
//                logger.warn("Can't restore parameters from string "+values+".", ex);
//                ok = false;
//            }
//            return ok;
//        }
//
//        public String toString() {
//            try {
//                DynObject obj = getDelegatedDynObject();
//                DynClass dynClass = obj.getDynClass();
//                DynField[] fields = dynClass.getDynFields();
//                StringWriter swriter = new StringWriter();
//                JsonWriter writer = new JsonWriter(swriter);
//                writer.setSerializeNulls(true);
//                writer.setLenient(true);
//                writer.setHtmlSafe(false);
//                writer.setIndent("");
//                writer.beginObject();
//                if (  fields != null && fields.length >0 ) {
//                    for ( int i = 0; i < fields.length; i++ ) {
//                        String fieldname = fields[i].getName();
//                        Object value = obj.getDynValue(fieldname);
//                        writer.name(fieldname);
//                        if( value == null ) {
//                            writer.value((String)null);
//                        } else {
//                            writer.value(value.toString());
//                        }
//                    }
//                }
//                writer.endObject();
//                writer.close();
//                return swriter.toString();
//            } catch (IOException ex) {
//                logger.warn("Can't serializar DataParamerters correctly.",ex);
//                return getDelegatedDynObject().toString();
//            }
//        }
//
        public void setDynValue(String name, Object value) {
		DelegatedDynObject delegated = getDelegatedDynObject();
		if (delegated.getDynClass().getDynField(name)!=null){
			delegated.setDynValue(name, value);	
		} else {
                    try {
                      throw new IllegalArgumentException(name);
                    } catch(IllegalArgumentException ex) {
			logger.warn("Attribute '"+name+"' is not defined in "
					+delegated.getDynClass().getFullName()+ " definition",ex);
                    }
		}
	}

	public void clear() {
		// TODO Delegar en el DynObject cuando tenga este servicio

		DynField[] fields =
				getDelegatedDynObject().getDynClass()
				.getDeclaredDynFields();

		for (int i = 0; i < fields.length; i++) {
			this.setDynValue(fields[i].getName(), fields[i].getDefaultValue());
		}
	}

	protected void copyValuesTo(AbstractDataParameters target) {
		// TODO Delegar en el DynObject cuando tenga este servicio
		DynField[] fields =
				getDelegatedDynObject().getDynClass()
				.getDynFields();


		for (int i = 0; i < fields.length; i++) {
			target.setDynValue(fields[i].getName(), this.getDynValue(fields[i]
					.getName()));
		}
	}

	public DataParameters getCopy() {
		// TODO Delegar en el DynObject cuando tenga este servicio
		AbstractDataParameters copy;
		try {
			copy = (AbstractDataParameters) this.getClass().newInstance();
		} catch (InstantiationException e) {
			throw new CopyParametersException("data parameters", e);
		} catch (IllegalAccessException e) {
			throw new CopyParametersException("data parameters", e);
		}
		this.copyValuesTo(copy);
		return copy;
	}

	public void saveToState(PersistentState state) throws PersistenceException {
		DynField[] fields =
				getDelegatedDynObject().getDynClass().getDynFields();

		for (int i = 0; i < fields.length; i++) {
		    if (fields[i].isPersistent()) {
		    	String name = fields[i].getName(); 
		    	Object value = this.getDynValue(fields[i].getName()); 
				state.set(name,value);
			}
		}
	}

	public void loadFromState(PersistentState state) throws PersistenceException {
		Iterator it = state.getNames();
		while (it.hasNext()) {
			String name = (String) it.next();
			this.setDynValue(name, state.get(name));
		}
	}

	public void delegate(DynObject dynObject) {
		getDelegatedDynObject().delegate(dynObject);

	}

	public DynClass getDynClass() {
		return getDelegatedDynObject().getDynClass();
	}

	public boolean hasDynValue(String name) {
		return getDelegatedDynObject().hasDynValue(name);
	}

	public void implement(DynClass dynClass) {
		getDelegatedDynObject().implement(dynClass);
	}

	public Object invokeDynMethod(String name, DynObject context)
			throws DynMethodException {
		return getDelegatedDynObject().invokeDynMethod(this, name, context);
	}

	public Object invokeDynMethod(int code, DynObject context)
			throws DynMethodException {
		return getDelegatedDynObject().invokeDynMethod(this, code, context);
	}

	public void validate() throws ValidateDataParametersException {
	    try {
            this.getDynClass().validate(this);
        } catch (DynObjectValidateException e) {
            throw new ValidateDataParametersException(e);
        }
	}

	/**
	 * Returns an instance of the {@link DynObject} to delegate to.
	 * 
	 * @return the delegate {@link DynObject}
	 */
	protected abstract DelegatedDynObject getDelegatedDynObject();

}
