/* gvSIG. Desktop Geographic Information System.
 *
 * Copyright  2007-2014 gvSIG Association
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.featureform.swing.impl;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Iterator;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JPanel;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.gvsig.featureform.swing.JFeatureForm;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.EditableFeature;
import org.gvsig.fmap.dal.feature.Feature;
import org.gvsig.fmap.dal.feature.FeatureQuery;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.paging.FeaturePagingHelper;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.dynform.AbortActionException;
import org.gvsig.tools.dynform.DynFormLocator;
import org.gvsig.tools.dynform.DynFormManager;
import org.gvsig.tools.dynform.JDynForm;
import org.gvsig.tools.dynform.JDynFormSet;
import org.gvsig.tools.dynform.JDynFormSet.JDynFormSetListener;
import org.gvsig.tools.dynobject.DynClass;
import org.gvsig.tools.dynobject.DynObject;
import org.gvsig.tools.exception.BaseException;
import org.gvsig.tools.i18n.I18nManager;
import org.gvsig.tools.locator.LocatorException;
import org.gvsig.tools.service.ServiceException;
import org.gvsig.tools.swing.api.ToolsSwingLocator;
import org.gvsig.tools.swing.api.windowmanager.WindowManager;
import org.gvsig.tools.swing.api.windowmanager.WindowManager.MODE;

/**
 * @author fdiaz
 *
 */
public class DefaultJFeatureForm implements JFeatureForm, JDynFormSetListener{

    private static final Logger LOGGER = LoggerFactory
        .getLogger(DefaultJFeatureForm.class);

    private static final int PAGE_SIZE = 500;
    private JPanel panel;
    private JDynFormSet formset;
    private FeatureStore store;
    private FeaturePagingHelper ph;
    private DynClass definition = null;

    /**
     *
     */
    public DefaultJFeatureForm() {
        this.panel = new JPanel(new BorderLayout());
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.attributeeditor.swing.JFeatureForm#setPreferredSize(java.awt
     * .Dimension)
     */
    public void setPreferredSize(Dimension dimension) {
        panel.setPreferredSize(dimension);
    }

    public JComponent asJComponent() {
        return this.panel;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.attributeeditor.swing.JFeatureForm#bind(org.gvsig.fmap.dal.
     * feature.FeatureStore)
     */
    public void bind(FeatureStore store) throws ServiceException, DataException {
        this.bind(store,store.getDefaultFeatureType());
    }
         
    public void bind(FeatureStore store, DynClass definition) throws ServiceException, DataException {
        if (this.store == store) {
            return;
        }

        if (formset != null) {
            this.panel.remove(formset.asJComponent());
            this.formset = null;
        }
        this.store = store;
        this.ph = null;
        this.definition = definition;
    }

    private JDynFormSet getFormset() throws ServiceException, DataException{
        if (this.formset == null){
            DynFormManager formManager = DynFormLocator.getDynFormManager();
            this.formset = formManager.createJDynFormSet(this.definition);

            this.formset.setLayoutMode(JDynForm.USE_SEPARATORS);
            this.formset.setReadOnly(false);
            this.formset.setAllowClose(true);
            this.formset.setAllowDelete(false);
            this.formset.setAllowNew(false);
            this.formset.setAllowSearch(false);
            this.formset.setAllowUpdate(true);
            this.formset.setAutosave(true);

            this.formset.addListener(this);
        }
        return this.formset;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.gvsig.attributeeditor.swing.JFeatureForm#setQuery(org.gvsig.fmap.
     * dal.feature.FeatureQuery)
     */
    public void setQuery(FeatureQuery query) throws LocatorException,
        BaseException {
        if (this.ph != null) {
            if (this.formset != null && this.formset.isAutosave()
                && this.formset.countValues() > 0) {
                if (!store.isEditing()) {
                    try {
                        store.edit();
                    } catch (DataException e1) {
                        throw new StoreEditException(e1, store.getName());
                    }
                }
                saveChanges(this.formset);
            }
        }
        if (this.formset == null){
            this.formset = getFormset();
            this.panel.add(this.formset.asJComponent(),BorderLayout.CENTER);
        }
        this.ph =
            DALLocator.getDataManager().createFeaturePagingHelper(store, query,
                PAGE_SIZE);
        FeatureSelection selection = (FeatureSelection) store.getSelection();
        selection.deselectAll();
        List features = ph.asList();
        for (Iterator iterator = features.iterator(); iterator.hasNext();) {
            Feature feature = (Feature) iterator.next();
            selection.select(feature);
        }
        if (features.size()>0) {
            this.showForm(MODE.TOOL);
        }

        this.formset.setValues(ph.asListOfDynObjects());
        this.formset.setReadOnly(false);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.gvsig.attributeeditor.swing.JFeatureForm#showForm()
     */
    public void showForm(MODE mode) throws ServiceException, DataException {
        if (this.formset == null) {
            this.formset = getFormset();
            this.panel.add(this.formset.asJComponent(),BorderLayout.CENTER);
            WindowManager winmgr = ToolsSwingLocator.getWindowManager();
            winmgr.showWindow(panel, store.getName(), mode);
        }
        this.panel.setVisible(true);

    }

    public void formMessage(String message) {
        // TODO Auto-generated method stub

    }

    public void formClose() {
        this.panel.setVisible(false);
    }

    public void formMovedTo(int currentPosition) throws AbortActionException {
        LOGGER.info("formMovedTo " + currentPosition);
    }

    class StoreEditException extends AbortActionException {

        /**
         *
         */
        private static final long serialVersionUID = -7682017811778577130L;

        public StoreEditException(Throwable cause, String storename) {
            super("Can't edit the store '%(storename)'", cause,
                "cant_edit_the store_XstorenameX", serialVersionUID);
            setValue("storename", storename);
        }
    }

    public void formBeforeSave(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formBeforeSave");
        if (!store.isEditing()) {
            try {
                store.edit();
            } catch (DataException e1) {
                throw new StoreEditException(e1, store.getName());
            }
        }
    }

    public void formBeforeNew(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formBeforeNew");
    }

    public void formBeforeDelete(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formBeforeDelete");
    }

    class GetFeatureException extends AbortActionException {

        /**
         *
         */
        private static final long serialVersionUID = -3545660152672760731L;

        public GetFeatureException(Throwable cause, int index) {
            super("Can't get the associated feature to index '%(index)'",
                cause, "cant_get_the_associated_feature_to_index_XindexX",
                serialVersionUID);
            setValue("index", new Integer(index));
        }
    }

    class UpdateFeatureException extends AbortActionException {

        /**
         *
         */
        private static final long serialVersionUID = 6473287361543242618L;

        public UpdateFeatureException(Throwable cause, int index,
            String storename) {
            super(
                "Can't update the feature '%(index)' in the store '%(storename)'",
                cause,
                "cant_update_the_feature_XindexX_in_the_store_XstorenameX",
                serialVersionUID);
            setValue("storename", storename);
            setValue("index", new Integer(index));
        }
    }

    public void formAfterSave(JDynFormSet dynformSet)
        throws AbortActionException {

        LOGGER.info("formAfterSave");
        saveChanges(dynformSet);
    }

    private void saveChanges(JDynFormSet dynformSet)
        throws AbortActionException {
        int index = dynformSet.getCurrentIndex();
        DynObject dynObject = dynformSet.get(index);
        dynformSet.getFormValues(dynObject);
        Feature feat = store.getFeature(dynObject);
        if (feat == null) {
            LOGGER.warn("Can't get the associated feature index " + index);
            I18nManager i18nManager = ToolsLocator.getI18nManager();
            dynformSet.message(i18nManager
                .getTranslation("error_saving_data_will_not_save"));
            throw new GetFeatureException(null, index);
        }

        try {
            ph.update((EditableFeature) feat);
        } catch (BaseException e) {
            throw new UpdateFeatureException(null, index, store.getName());
        }
    }

    public void saveChanges() throws BaseException {
        if (this.formset != null && this.formset.countValues() > 0 ) {
            if (store != null && !store.isEditing()) {
                try {
                    store.edit();
                } catch (DataException e1) {
                    LOGGER.warn("Can't edit the store " + store.getName());
                    throw new StoreEditException(e1, store.getName());
                }
            }
            this.saveChanges(this.formset);
        }
    }

    public void formAfterNew(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formAfterNew");
    }

    public void formAfterDelete(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formAfterDelete");
    }

    public void formBeforeSearch(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formBeforeSearch");
    }

    public void formAfterSearch(JDynFormSet dynformSet)
        throws AbortActionException {
        LOGGER.info("formAfterSearch");
    }

    public long getDataSetSize() {
        if (this.ph != null) {
            return ph.getTotalSize();
        }
        return 0;
    }


}
