/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */

package org.gvsig.fmap.mapcontext.rendering.legend.styling;

import org.gvsig.tools.persistence.Persistent;


/**
 * @author   jaume dominguez faus - jaume.dominguez@iver.es
 */
public interface IZoomConstraints extends Persistent {
	/**
	 * Uses the same constraints than the layer. Which is the same than no constraints.
	 */
	public static final int DEFINED_BY_THE_LAYER = 0;

	/**
	 * The user defines when the labes will be drawn.
	 * Notice that if the layer is not drawn it does not take effect.
	 */
	public static final int DEFINED_BY_THE_USER = 1;

	/**
	 * Sets the mode of the zoom constraints.
	 * @param mode, int one of DEFINED_BY_THE_LAYER or DEFINED_BY_THE_USER.
	 */
	public abstract void setMode(int mode);

	/**
	 * Returns the max scale limit of the constraints.
	 * @return   float, max limit
	 * @uml.property  name="maxScale"
	 */
	public abstract long getMaxScale();

	/**
	 * Sets the max scale limit of the constraints
	 * @param   maxScale, float
	 * @uml.property  name="maxScale"
	 */
	public abstract void setMaxScale(long maxScale);

	/**
	 * Returns the min scale limit of the constraints.
	 * @return   float, min limit
	 * @uml.property  name="minScale"
	 */
	public abstract long getMinScale();

	/**
	 * Sets the min scale limit of the constraints
	 * @param   minScale, float
	 * @uml.property  name="minScale"
	 */
	public abstract void setMinScale(long minScale);

	/**
	 * Returns <b>true</b> if the mode is DEFINED_BY_THE_USER.
	 * @return
	 */
	public abstract boolean isUserDefined();

	/**
	 * Returns <b>true</b> if the mode is DEFINED_BY_THE_USER.
	 * @return
	 */
	public abstract boolean isLayerDefined();

}
