/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.i18n.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import junit.framework.TestCase;

/**
 * @author cesar
 *
 */
public class TestProperties extends TestCase {
	static final String fileName = "TestProperties-file";
	InputStream is;
	OutputStream os;
	
	public void initOutputStream() {
		try {
			os = new FileOutputStream(fileName);
		} catch (FileNotFoundException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
	}
	
	public void discardOutputStream() {
		try {
			os.close();
			File file = new File(fileName);
			file.delete();
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
	}
	
	public void initInputStream() {
		try {
			is = new FileInputStream(fileName);
		} catch (FileNotFoundException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
	}
	
	public void discardInputStream() {
		try {
			is.close();
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
	}
	
	public void testOrderedPropertiesBasic() {
		// write file
		OrderedProperties props1 = new OrderedProperties();
		props1.setProperty("prueba", "hola");
		props1.setProperty("prueba2", "holas");
		props1.setProperty("abrir", "cerrar");
		props1.setProperty("puerta", "hembrilla");
		props1.setProperty("12libros", "quijote");
		props1.setProperty("cauto", "\u20acuropeo");
		props1.setProperty("confli=cto", "amigo");
		props1.setProperty("  confli =  :cto mayor ", " pequeo conflicto");
		initOutputStream();
		try {
			props1.store(os, "header");
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
		
		// read file
		OrderedProperties props2 = new OrderedProperties();
		initInputStream();
		try {
			props2.load(is);
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
		TestCase.assertEquals("\u20acuropeo", props2.getProperty("cauto"));
		TestCase.assertEquals("amigo", props2.getProperty("confli=cto"));
		TestCase.assertEquals(" pequeo conflicto", props2.getProperty("  confli =  :cto mayor "));
		TestCase.assertEquals("hola", props2.getProperty("prueba"));
		TestCase.assertEquals("holas", props2.getProperty("prueba2"));
		TestCase.assertEquals("cerrar", props2.getProperty("abrir"));
		TestCase.assertEquals("hembrilla", props2.getProperty("puerta"));
		TestCase.assertEquals("quijote", props2.getProperty("12libros"));
		discardInputStream();
		discardOutputStream();
	}

	
	public void testOrderedProperties() {
		// write file
		OrderedProperties props1 = new OrderedProperties();
		props1.setProperty("prueba", "hola");
		props1.setProperty("prueba2", "holas");
		props1.setProperty("cauto", "\u20acuropeo");
		props1.setProperty("confli=cto", "amigo");
		props1.setProperty("  conflis =  :cto mayor ", " pequeo conflicto");
		props1.setProperty("abrir", "cerrar");
		props1.setProperty("puerta", "hembrilla");
		props1.setProperty("12libros", "quijote");
		initOutputStream();
		try {
			props1.store(os, "header", "8859_15");
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
		
		// read file
		OrderedProperties props2 = new OrderedProperties();
		initInputStream();
		try {
			props2.load(is, "8859_15");
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
		// estos deberan salir igual
		TestCase.assertEquals("hola", props2.getProperty("prueba"));
		TestCase.assertEquals("holas", props2.getProperty("prueba2"));
		TestCase.assertEquals("cerrar", props2.getProperty("abrir"));
		TestCase.assertEquals("hembrilla", props2.getProperty("puerta"));
		TestCase.assertEquals("quijote", props2.getProperty("12libros"));
		// con estos en cambio no obtenemos los mismos valores que escribimos anteriormente. Es normal, porque incluimos
		// caracteres especiales como ":" y "=". Para manejarlos correctamente necesitamos usas los mtodos load/store
		// sin encoding, los cuales s escapan los caracteres especiales
		TestCase.assertEquals("\u20acuropeo", props2.getProperty("cauto"));
		TestCase.assertEquals("cto=amigo", props2.getProperty("confli"));
		TestCase.assertEquals(":cto mayor = pequeo conflicto", props2.getProperty("conflis"));

		discardInputStream();
		discardOutputStream();
		
		// write file
		props1 = new OrderedProperties();
		props1.setProperty("Prueba", "hola");
		props1.setProperty("prueba2", "holas");
		props1.setProperty("cauto", "\u20acuropeo");
		props1.setProperty("confli=cto", "amigo");
		props1.setProperty("  conflis =  :cto mayor ", " pequeo conflicto");
		props1.setProperty("abrir", "cerrar");
		props1.setProperty("puerta", "hembrilla");
		props1.setProperty("12libros", "quijote");
		initOutputStream();
		try {
			props1.store(os, "header", "UTF8");
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
		
		// read file
		props2 = new OrderedProperties();
		initInputStream();
		try {
			props2.load(is, "UTF8");
		} catch (IOException e) {
			TestCase.fail(e.getLocalizedMessage());
		}
		// estos deberan salir igual
		TestCase.assertEquals("hola", props2.getProperty("Prueba"));
		TestCase.assertEquals("holas", props2.getProperty("prueba2"));
		TestCase.assertEquals("cerrar", props2.getProperty("abrir"));
		TestCase.assertEquals("hembrilla", props2.getProperty("puerta"));
		TestCase.assertEquals("quijote", props2.getProperty("12libros"));
		// con estos en cambio no obtenemos los mismos valores que escribimos anteriormente. Es normal, porque incluimos
		// caracteres especiales como ":" y "=". Para manejarlos correctamente necesitamos usas los mtodos load/store
		// sin encoding, los cuales s escapan los caracteres especiales
		TestCase.assertEquals("\u20acuropeo", props2.getProperty("cauto"));
		TestCase.assertEquals("cto=amigo", props2.getProperty("confli"));
		TestCase.assertEquals(":cto mayor = pequeo conflicto", props2.getProperty("conflis"));

		discardInputStream();
		discardOutputStream();
	}
	
}
