/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.cresques.impl.cts.gt2;

import org.cresques.cts.UTM;
import org.geotools.cs.AxisInfo;
import org.geotools.cs.Projection;
import org.geotools.units.Unit;
import org.gvsig.fmap.crs.CRSFactory;
import org.opengis.referencing.FactoryException;


/**
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class CSUTM extends CoordSys implements UTM {
    public CSUTM(CSDatum datum, int zone) {
        super(datum);
        init(datum, zone, "N");
    }
    
    public CSUTM(CSDatum datum, int zone, String ns) {
        super(datum);
        init(datum, zone, ns);
    }
 
    public void init(CSDatum datum, int zone, String ns) {
        Unit linearUnit = Unit.METRE;

        javax.media.jai.ParameterList params = csFactory.createProjectionParameterList("Transverse_Mercator");
        params.setParameter("semi_major",
                            datum.getDatum().getEllipsoid().getSemiMajorAxis());
        params.setParameter("semi_minor",
                            datum.getDatum().getEllipsoid().getSemiMinorAxis());
        params.setParameter("central_meridian", (double) ((zone * 6) - 183));
        params.setParameter("latitude_of_origin", 0.0);
        params.setParameter("scale_factor", 0.9996);
        params.setParameter("false_easting", 500000.0);
        if (ns.toUpperCase().compareTo("S") == 0)
        	params.setParameter("false_northing", 10000000.0);
        else
        	params.setParameter("false_northing", 0.0);

        try {
            Projection projection = csFactory.createProjection("UTM" + zone,
                                                               "Transverse_Mercator",
                                                               params);
            projCS = csFactory.createProjectedCoordinateSystem(projection.getName()
                                                                         .toString(),
                                                               geogCS,
                                                               projection,
                                                               linearUnit,
                                                               AxisInfo.X,
                                                               AxisInfo.Y);
        } catch (FactoryException e) {
            // TODO Bloque catch generado automticamente
            e.printStackTrace();
        }
    }

    public double getScale(double minX, double maxX, double w, double dpi) {
        double scale = super.getScale(minX, maxX, w, dpi);

        if (projCS != null) { // Es geogrfico; calcula la escala.
            scale = ((maxX - minX) * // metros
                    (dpi / 2.54 * 100.0)) / // px / metro
                    w; // pixels
        }

        return scale;
    }

    public String toString() {
        return projCS.toString();
    }
    
    public Object clone() throws CloneNotSupportedException {
        return CRSFactory.getCRS( this.getFullCode() );
     }

}
