/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.cresques.impl.geo;

import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import java.util.Iterator;
import java.util.Vector;

import org.cresques.geo.ViewPortData;


/**
 * Clase que representa un polgono 2D
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
public class Polygon2D extends Vector {
    GeneralPath gp = null;

    public Polygon2D() {
        super();
        gp = null;
    }

    /**
     * Aade un vertice al po?gono
     * @param pt        punto 2D que representa el vertice aadido
     */
    public void addPoint(Point2D pt) {
        super.add(pt);
    }

    /**
     * Dibuja el polgono
     * @param g        Graphics sobre el que dibuja
     * @param vp        ViewPort con la vista
     */
    public void draw(Graphics2D g, ViewPortData vp) {
        newGP(vp);
        g.draw(gp);

        //g.draw(new Line2D.Double(pt,pt0));
    }

    /**
     *
     * @param g
     * @param vp
     */
    public void fill(Graphics2D g, ViewPortData vp) {
        newGP(vp);
        g.fill(gp);
    }

    /**
     *
     * @param vp
     */
    private void newGP(ViewPortData vp) {
        //if (gp != null) return;
        gp = new GeneralPath();

        Point2D pt0 = null;
        Point2D pt = null;
        Point2D pt1 = null;
        Point2D.Double ptTmp = new Point2D.Double(0.0, 0.0);
        Iterator iter = iterator();

        while (iter.hasNext()) {
            pt1 = (Point2D) iter.next();
            vp.mat.transform(pt1, ptTmp);

            if (pt0 == null) {
                pt0 = ptTmp;
                gp.moveTo((float) ptTmp.getX(), (float) ptTmp.getY());
            } else {
                gp.lineTo((float) ptTmp.getX(), (float) ptTmp.getY());
            }
        }

        gp.closePath();
    }
}
