/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.remoteclient.wcs;

import java.awt.geom.Rectangle2D;

import org.gvsig.remoteclient.RemoteClientStatus;
/**
 * Describes the status of a WCSclient, so it adds to the Remote client status
 * a list of layers, a list of layer styles, the extent of the map.
 * Provides the functionality to modify these lists.
 * 
 * @author jaume domnguez faus - jaume.dominguez@iver.es
 */
public class WCSStatus extends RemoteClientStatus {
	//private String		resolution		= null; TODO do I need it?
	private Rectangle2D	bBox			= null;
	private String		coverageName	= null;
	private String		onlineResource	= null;
	private String		depth			= null;
	private String		times			= null;
	private String		parameters		= null;
	private String		message			= null;
	
	/**
	 * Sets the Bounding Box that is going to be requested to the server.
	 * @param bBox, Rectangle2D containing the edges of the bounding box
	 */
	public void setExtent(Rectangle2D bBox) {
		this.bBox = bBox;
	}
	
	/**
	 * Sets the name of the coverage requested to the server.
	 * @param coverageName, String containing the name of the requesting coverage
	 */
	public void setCoveraName(String coverageName) {
		this.coverageName = coverageName;
	}

	/**
	 * Sets the parameter string. The parameter string is the part of the request 
	 * containing the value for the axis as described in the DescribeCoverage
	 * document.
	 * @param parameters
	 */
	public void setParameters(String parameters) {
		this.parameters = parameters;
	}

	/**
	 * Will set the address of the host that serves the following request
	 * @param onlineResources
	 */
	public void setOnlineResource(String onlineResources) {
		this.onlineResource = onlineResources;
	}
	
	/**
	 * Sets the depth. The depth is the third dimension for the size. 
	 * @param depth, String
	 */
	public void setDepth(String depth) {
		this.depth = depth;
	}

	/**
	 * Returns the address host that will serve the request 
	 * @return String
	 */
	public String getOnlineResource() {
		return onlineResource;
	}

	/**
	 * Return the name of the coverage requested to the server.
	 * @return String containing the name of the requesting coverage
	 */
	public String getCoverageName() {
		return this.coverageName;
	}
	
	/**
	 * Returns the Bounding Box that is going to be requested to the server.
	 * @return Rectangle2D containing the edges of the bounding box
	 */
	public Rectangle2D getExtent() {
		return this.bBox;
	}

	/**
	 * Returns the depth of the next request.
	 * @return String
	 */
	public String getDepth() {
		return depth;
	}

	/**
	 * Returns the value for the time parameter that will be used in the next request
	 * @return String
	 */
	public String getTime() {
		return times;
	}
	
	/**
	 * Returns the parameter string.
	 * The parameter string is the part of the request containing the value for the
	 * axis as described in the DescribeCoverage.
	 * @return String
	 */
	public String getParameters() {
		return parameters;
	}

	/**
	 * Sets the value(s) for the time that will be used in the next request. 
	 * @param times
	 */
	public void setTime(String times) {
		this.times = times;
	}

	/**
	 * Returns the message string
	 * @return String
	 */
	public String getMessage() {
		return message;
	}
	
	/**
	 * Sets the message string 
	 * @param message
	 */
	public void setMessage(String message) {
		this.message = message;
	}
	
}
