/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.andami.installer.packagebuilder;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.File;
import java.net.URL;

import javax.swing.JPanel;
import org.gvsig.andami.installer.packagebuilder.options.OutputOption;
import org.gvsig.andami.installer.packagebuilder.options.PackageInfoOption;
import org.gvsig.andami.installer.packagebuilder.options.ProgressOption;
import org.gvsig.andami.installer.packagebuilder.options.SelectFolderToPackagingOption;
import org.gvsig.andami.installer.translations.TranslationsInstallerFactory;

import org.gvsig.gui.beans.wizard.WizardPanel;
import org.gvsig.gui.beans.wizard.WizardPanelActionListener;
import org.gvsig.gui.beans.wizard.WizardPanelWithLogo;
import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.installer.lib.api.PackageInfo;
import org.gvsig.tools.ToolsLocator;
import org.gvsig.tools.locator.LocatorException;

public class BasePackageWizard extends JPanel implements PackageWizard {

    private static final long serialVersionUID = 9205891710214122265L;

    private WizardPanelWithLogo wizardPanelWithLogo = null;

    private PackageInfo packageInfo = null;
    private File folderToPackaging = null;
    private WizardPanelActionListener wizardListenerAdapter = null;

    private OutputOption outputOption = null;
    private OptionPanel packageInfoOption = null;
    private OptionPanel progressOption = null;
    private SelectFolderToPackagingOption selectFolderToPackagingOption = null;

    public BasePackageWizard(String packageType) throws LocatorException {
        this.wizardPanelWithLogo = new WizardPanelWithLogo();

        setCancelButtonEnabled(true);
        setFinishButtonEnabled(false);

        packageInfo = ToolsLocator.getPackageManager().createPackageInfo();
        packageInfo.setType(packageType);
        
        this.initOptionPanels();

        // Adding the listeners
        wizardPanelWithLogo.setWizardListener(this);

        this.setLayout(new BorderLayout());
        this.add(wizardPanelWithLogo, BorderLayout.CENTER);
        this.setPreferredSize(new Dimension(700, 550));
    }
    
    protected void addPackageInfoOptionPanel() {
        if ( this.packageInfoOption == null ) {
            this.packageInfoOption = new PackageInfoOption(this);
        }
        this.wizardPanelWithLogo.addOptionPanel(this.packageInfoOption);
    }

    protected void addOutputOptionPanel() {
        if ( this.outputOption == null ) {
            this.outputOption = new OutputOption(this);
        }
        this.wizardPanelWithLogo.addOptionPanel(this.outputOption);
    }

    protected void addProgressOptionPanel() {
        if ( this.progressOption == null ) {

            this.progressOption = new ProgressOption(this);
        }
        this.wizardPanelWithLogo.addOptionPanel(this.progressOption);
    }
    
    protected void addSelectFolderToPackagingOption(String message) {
        if( this.selectFolderToPackagingOption == null ) {
            this.selectFolderToPackagingOption = new SelectFolderToPackagingOption(this, message);
        }
        this.wizardPanelWithLogo.addOptionPanel(this.selectFolderToPackagingOption);
    }

    protected void addDefaultOptionPanels() {
        this.addPackageInfoOptionPanel();
        this.addOutputOptionPanel();
        this.addProgressOptionPanel();
    }

    protected void initOptionPanels() {
        this.addDefaultOptionPanels();
    }
    
    public void setNextButtonEnabled(boolean isEnabled) {
        wizardPanelWithLogo.setNextButtonEnabled(isEnabled);
    }

    public void setCancelButtonEnabled(boolean isEnabled) {
        wizardPanelWithLogo.setCancelButtonEnabled(isEnabled);
    }

    public void setFinishButtonEnabled(boolean isEnabled) {
        wizardPanelWithLogo.setFinishButtonEnabled(isEnabled);
    }

    public void setBackButtonEnabled(boolean isEnabled) {
        wizardPanelWithLogo.setBackButtonEnabled(isEnabled);
    }

    public WizardPanelActionListener getWizardPanelActionListener() {
        if ( this.wizardListenerAdapter == null ) {
            this.wizardListenerAdapter = new WizardPanelActionListener() {
                public void finish(WizardPanel wizardPanel) {
                    BasePackageWizard.this.setVisible(false);
                }

                public void cancel(WizardPanel wizardPanel) {
                    BasePackageWizard.this.setVisible(false);
                }
            };
        }
        return this.wizardListenerAdapter;
    }

    public void setWizardPanelActionListener(
            WizardPanelActionListener wizardActionListener) {
        // TODO Auto-generated method stub
        // this.wizardListenerAdapter = wizardActionListener;
        throw new RuntimeException("Esto falta por ver que hace");
    }

    public void setFolderToPackaging(File file) {
        this.folderToPackaging = file;
        if( this.selectFolderToPackagingOption!=null ) {
            this.selectFolderToPackagingOption.setCurrentFolder(file);
        }
        File pinfo = new File(file, "package.info");
        if ( pinfo.exists() ) {
            try {
                ToolsLocator.getPackageManager().readPacakgeInfo(
                        (org.gvsig.tools.packageutils.PackageInfo) packageInfo, pinfo);
                packageInfo.setType(TranslationsInstallerFactory.PROVIDER_NAME);
            } catch (Exception e) {
                /*
                 * Do nothing, packageInfo remains empty
                 * form will be empty
                 * 
                 */
            }
        }

    }

    public File getFolderToPackaging() {
        return this.folderToPackaging;
    }

    public PackageInfo getPackageInfo() {
        return this.packageInfo;
    }

    public URL getDownloadURL() {
        return this.outputOption.getDownloadURL();
    }

    public File getPackageIndexFile() {
        return this.outputOption.getPackageIndexFile();
    }

    public boolean shouldCreateIndex() {
        return this.outputOption.shouldCreateIndex();
    }

    public File getPackageFile() {
        return this.outputOption.getPackageFile();
    }

    public void addOptionPanel(OptionPanel optionPanel) {
        this.wizardPanelWithLogo.addOptionPanel(optionPanel);
    }
}
