/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.andami.messages;

import java.util.EventListener;


/**
 * Interfaz implementada por las extensiones que quieran informar del estado de
 * su ejecucin. Por ejemplo un extension que tarda mucho en ejecutarse puede
 * implementar esta interfaz y aadirse como ProgressListener de la
 * aplicacion. De esta manera la aplicacin le pedir su estado a intervalos
 * regulares de tiempo y mostrar esta informacin en la barra de estado
 *
 * @author Fernando Gonzlez Corts
 */
public interface ProgressListener extends EventListener {
    /**
     * Mtodo invocado a intervalos regulares por la aplicacin y en el que se
     * debe devolver el estado de una supuesta ejecucin
     *
     * @return Cadena que se enva a la aplicacin
     */
    public String getProgress();

    /**
     * Devuelve un porcentaje que indica el progreso de la tarea
     *
     * @return nmero del 0 al 100
     */
    public int getProgressValue();
}
