package org.gvsig.andami.ui;
import java.awt.BorderLayout;
import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;


public class DisablePluginsConflictingLayoutPanel extends JPanel
{
   /**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
   @SuppressWarnings("rawtypes")
   public JList pluginList = new JList();
   public JButton buttonContinue = new JButton();
   public JButton buttonClose = new JButton();
   public JLabel lblConflict = new JLabel();
   public JLabel lblSelectPluginToDisable = new JLabel();
   public JLabel lblClickContinue = new JLabel();
   public JLabel lblClickClose = new JLabel();

   /**
    * Default constructor
    */
   public DisablePluginsConflictingLayoutPanel()
   {
      initializePanel();
   }

   /**
    * Adds fill components to empty cells in the first row and first column of the grid.
    * This ensures that the grid spacing will be the same as shown in the designer.
    * @param cols an array of column indices in the first row where fill components should be added.
    * @param rows an array of row indices in the first column where fill components should be added.
    */
   void addFillComponents( Container panel, int[] cols, int[] rows )
   {
      Dimension filler = new Dimension(10,10);

      boolean filled_cell_11 = false;
      CellConstraints cc = new CellConstraints();
      if ( cols.length > 0 && rows.length > 0 )
      {
         if ( cols[0] == 1 && rows[0] == 1 )
         {
            /** add a rigid area  */
            panel.add( Box.createRigidArea( filler ), cc.xy(1,1) );
            filled_cell_11 = true;
         }
      }

      for( int index = 0; index < cols.length; index++ )
      {
         if ( cols[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(cols[index],1) );
      }

      for( int index = 0; index < rows.length; index++ )
      {
         if ( rows[index] == 1 && filled_cell_11 )
         {
            continue;
         }
         panel.add( Box.createRigidArea( filler ), cc.xy(1,rows[index]) );
      }

   }

   /**
    * Helper method to load an image file from the CLASSPATH
    * @param imageName the package and name of the file to load relative to the CLASSPATH
    * @return an ImageIcon instance with the specified image file
    * @throws IllegalArgumentException if the image resource cannot be loaded.
    */
   public ImageIcon loadImage( String imageName )
   {
      try
      {
         ClassLoader classloader = getClass().getClassLoader();
         java.net.URL url = classloader.getResource( imageName );
         if ( url != null )
         {
            ImageIcon icon = new ImageIcon( url );
            return icon;
         }
      }
      catch( Exception e )
      {
         e.printStackTrace();
      }
      throw new IllegalArgumentException( "Unable to load image: " + imageName );
   }

   /**
    * Method for recalculating the component orientation for 
    * right-to-left Locales.
    * @param orientation the component orientation to be applied
    */
   public void applyComponentOrientation( ComponentOrientation orientation )
   {
      // Not yet implemented...
      // I18NUtils.applyComponentOrientation(this, orientation);
      super.applyComponentOrientation(orientation);
   }

   public JPanel createPanel()
   {
      JPanel jpanel1 = new JPanel();
      FormLayout formlayout1 = new FormLayout("FILL:DEFAULT:NONE,FILL:DEFAULT:GROW(1.0),FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE,FILL:DEFAULT:NONE","CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:GROW(1.0),CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE,CENTER:DEFAULT:NONE");
      CellConstraints cc = new CellConstraints();
      jpanel1.setLayout(formlayout1);

      lblConflict.setName("lblConflict");
      lblConflict.setText("Some of plugins installed conflict with each other.");
      jpanel1.add(lblConflict,new CellConstraints(2,2,4,1,CellConstraints.FILL,CellConstraints.DEFAULT));

      pluginList.setName("PluginList");
      JScrollPane jscrollpane1 = new JScrollPane();
      jscrollpane1.setViewportView(pluginList);
      jscrollpane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
      jscrollpane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      jpanel1.add(jscrollpane1,new CellConstraints(2,7,4,1,CellConstraints.FILL,CellConstraints.FILL));

      buttonContinue.setActionCommand("Continue");
      buttonContinue.setText("Continue");
      jpanel1.add(buttonContinue,cc.xy(5,9));

      buttonClose.setActionCommand("Close");
      buttonClose.setName("buttonClose");
      buttonClose.setText("Close");
      jpanel1.add(buttonClose,cc.xy(3,9));

      lblSelectPluginToDisable.setName("lblSelectPluginToDisable");
      lblSelectPluginToDisable.setText("Select the plugins that you want to disable and click the continue button.");
      jpanel1.add(lblSelectPluginToDisable,cc.xywh(2,3,4,1));

      lblClickContinue.setName("lblClickContinue");
      lblClickContinue.setText("You can click on continue button directly if you want to disable any plugins.");
      jpanel1.add(lblClickContinue,cc.xywh(2,4,4,1));

      lblClickClose.setName("lblClickClose");
      lblClickClose.setText("Or click the close button to close the application.");
      jpanel1.add(lblClickClose,cc.xywh(2,5,4,1));

      addFillComponents(jpanel1,new int[]{ 1,2,3,4,5,6 },new int[]{ 1,2,3,4,5,6,7,8,9,10 });
      return jpanel1;
   }

   /**
    * Initializer
    */
   protected void initializePanel()
   {
      setLayout(new BorderLayout());
      add(createPanel(), BorderLayout.CENTER);
   }


}
