/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.andami.ui.wizard;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.plugins.status.IUnsavedData;
import org.gvsig.andami.ui.mdiManager.IWindow;
import org.gvsig.andami.ui.mdiManager.WindowInfo;
import org.gvsig.gui.beans.swing.JButton;


/**
 * Ask for elements to save before application exit
 *
 *
 */
public class UnsavedDataPanel extends JPanel implements IWindow, ActionListener {
	/**
	 *
	 */
	private static final long serialVersionUID = -4745219917358767905L;


	private static ImageIcon blanckIcon = null;
	private JPanel pResources = null;
	private JPanel pButtons = null;
	private JScrollPane pScrollList = null;
	private myList list = null;
	private JLabel lblDescription = null;
	private JButton botSave = null;
	private JButton botDiscard = null;
	private JButton botDontExit = null;
	private JPanel pActionButtons = null;
	private JPanel pSelectionButtons = null;
	private JButton botSelectAll = null;
	private JButton botDeselectAll = null;

	private ArrayList listeners = new ArrayList();
	private JLabel lblResourceDescription = null;
	private IUnsavedData[] unsavedDataList;
	private String windowTitle = PluginServices.getText(this, "save_resources");
	private String prompt = PluginServices.getText(this, "select_resources_to_save_before_exit");



	/**
	 * This is the default constructor
	 */
	public UnsavedDataPanel(IUnsavedData[] unsavedDatalist) {
		super();
		initialize();
		this.setUnsavedDataArray(unsavedDatalist);
	}

	public UnsavedDataPanel(String prompt) {
        super();
        initialize();
        this.setUnsavedDataArray(new IUnsavedData[0]);
        this.prompt = prompt;
    }


	/**
	 * This method initializes this
	 *
	 * @return void
	 */
	private void initialize() {
		BorderLayout borderLayout = new BorderLayout();
		borderLayout.setHgap(5);
		borderLayout.setVgap(5);
		lblDescription = new JLabel();
		lblDescription.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
		lblDescription.setText(prompt);
		lblDescription.setPreferredSize(new Dimension(497, 30));
		lblDescription.setName("lblDescription");
		this.setLayout(borderLayout);
		this.setSize(396, 272);
		this.add(getPResources(), java.awt.BorderLayout.CENTER);
		this.add(getPButtons(), java.awt.BorderLayout.SOUTH);
		this.add(lblDescription, BorderLayout.NORTH);
	}

	/**
	 * This method initializes pResources
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPResources() {
		if (pResources == null) {
			pResources = new JPanel();
			pResources.setLayout(new BorderLayout());
			pResources.add(getPScrollList(), BorderLayout.CENTER);
			// pResources.add(getLblResourceDescription(), BorderLayout.SOUTH);
		}
		return pResources;
	}


	/**
	 * This method initializes lblResourceDescription
	 *
	 * @return javax.swing.JLabel
	 */
	private JLabel getLblResourceDescription() {
		if (lblResourceDescription == null) {
			lblResourceDescription = new JLabel();
			lblResourceDescription.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 0));
			lblResourceDescription.setText("");
			lblResourceDescription.setPreferredSize(new Dimension(38, 50));
			lblResourceDescription.setName("lblResourceDescription");
		}
		return lblResourceDescription;
	}


	/**
	 * This method initializes pButtons
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPButtons() {
		if (pButtons == null) {
			BorderLayout borderLayout2 = new BorderLayout();
			borderLayout2.setHgap(5);
			borderLayout2.setVgap(5);
			pButtons = new JPanel();
			pButtons.setLayout(borderLayout2);
			pButtons.add(getPSelectionButtons(), BorderLayout.NORTH);
			pButtons.add(getPActionButtons(), BorderLayout.SOUTH);
		}
		return pButtons;
	}

	/**
	 * This method initializes pScrollList
	 *
	 * @return javax.swing.JScrollPane
	 */
	private JScrollPane getPScrollList() {
		if (pScrollList == null) {
			pScrollList = new JScrollPane();
			pScrollList.setPreferredSize(new java.awt.Dimension(350,350));
			pScrollList.setViewportView(getList());
		}
		return pScrollList;
	}

	/**
	 * This method initializes list
	 *
	 * @return javax.swing.JList
	 */
	private myList getList() {
		if (list == null) {
			list = new myList();
			list.setBackground(Color.white);
		}
		return list;
	}

	/**
	 * This method initializes botSave
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBotSave() {
		if (botSave == null) {
			botSave = new JButton();
			botSave.setName("botSave");
			botSave.setToolTipText(PluginServices.getText(this, "save_selected_resources_and_exit"));
			botSave.setText(PluginServices.getText(this, "ok"));
			botSave.setActionCommand("Accept");
			botSave.addActionListener(this);
		}
		return botSave;
	}

	/**
	 * This method initializes botDiscard
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBotDiscard() {
		if (botDiscard == null) {
			botDiscard = new JButton();
			botDiscard.setName("botDiscard");
			botDiscard.setToolTipText(PluginServices.getText(this, "discard_changes_and_exit"));
			botDiscard.setText(PluginServices.getText(this, "discard_changes"));
			botDiscard.setActionCommand("Discard");
			botDiscard.addActionListener(this);
		}
		return botDiscard;
	}

	/**
	 * This method initializes botDontExit
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBotDontExit() {
		if (botDontExit == null) {
			botDontExit = new JButton();
			botDontExit.setName("botDontExit");
			botDontExit.setToolTipText(PluginServices.getText(this, "cancel_the_application_termination"));
			botDontExit.setText(PluginServices.getText(this, "cancelar"));
			botDontExit.setActionCommand("Cancel");
			botDontExit.addActionListener(this);
		}
		return botDontExit;
	}

	/**
	 * This method initializes pActionButtons
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPActionButtons() {
		if (pActionButtons == null) {

		    GridLayout gl = new GridLayout(1, 3, 8, 0);
		    /*
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setAlignment(FlowLayout.RIGHT);
			*/
			pActionButtons = new JPanel();
			pActionButtons.setBorder(BorderFactory.createEmptyBorder(8,8,8,8));
			pActionButtons.setLayout(gl);
			pActionButtons.setName("pActionButtons");
			pActionButtons.add(getBotSave());
			pActionButtons.add(getBotDiscard());
			pActionButtons.add(getBotDontExit());
		}
		return pActionButtons;
	}

	/**
	 * This method initializes pSelectionButtons
	 *
	 * @return javax.swing.JPanel
	 */
	private JPanel getPSelectionButtons() {
		if (pSelectionButtons == null) {
		    GridLayout gl = new GridLayout(2, 3, 8, 8);
		    /*
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(FlowLayout.LEFT);
			*/
			pSelectionButtons = new JPanel();
			pSelectionButtons.setBorder(BorderFactory.createEmptyBorder(5,8,8,8));
			pSelectionButtons.setLayout(gl);
			pSelectionButtons.setName("pSelectionButtons");
			pSelectionButtons.add(getBotSelectAll());
			pSelectionButtons.add(new JLabel());
			pSelectionButtons.add(new JLabel());
			// new row
			pSelectionButtons.add(getBotDeselectAll());
			pSelectionButtons.add(new JLabel());
            pSelectionButtons.add(new JLabel());

		}
		return pSelectionButtons;
	}

	/**
	 * This method initializes botSelectAll
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBotSelectAll() {
		if (botSelectAll == null) {
			botSelectAll = new JButton();
			botSelectAll.setText(PluginServices.getText(this, "select_all"));
			botSelectAll.setName("botSelectAll");
			botSelectAll.setToolTipText(PluginServices.getText(this, "select_all_resources"));
			botSelectAll.setActionCommand("SelectAll");
			botSelectAll.addActionListener(this);
		}
		return botSelectAll;
	}

	/**
	 * This method initializes botDeselectAll
	 *
	 * @return javax.swing.JButton
	 */
	private JButton getBotDeselectAll() {
		if (botDeselectAll == null) {
			botDeselectAll = new JButton();
			botDeselectAll.setName("botDeselectAll");
			botDeselectAll.setToolTipText(PluginServices.getText(this, "deselect_all_resources"));
			botDeselectAll.setText(PluginServices.getText(this, "deselect_all"));
			botDeselectAll.setActionCommand("DeselectAll");
			botDeselectAll.addActionListener(this);
		}
		return botDeselectAll;
	}

	public WindowInfo getWindowInfo() {
		WindowInfo wi = new WindowInfo(WindowInfo.MODALDIALOG
				| WindowInfo.RESIZABLE
				| WindowInfo.MAXIMIZABLE);
		wi.setTitle(windowTitle);
		wi.setHeight(270);
		wi.setWidth(450);
		return wi;
	}

	public void setWindowTitle(String windowTitle) {
		this.windowTitle = windowTitle;
	}

	public void setCancelText(String text, String tooltipText) {
		getBotDontExit().setToolTipText(tooltipText);
		getBotDontExit().setText(text);
	}

	public void setDiscardText(String text, String tooltipText) {
		getBotDiscard().setToolTipText(tooltipText);
		getBotDiscard().setText(text);
	}

	public void setAcceptText(String text, String tooltipText) {
		getBotSave().setToolTipText(tooltipText);
		getBotSave().setText(text);
	}

	public void setHeaderText(String text) {
		lblDescription.setText(text);
	}

	/**
	 * Register a listener to call
	 *
	 * @param a UnsavedDataPanelListener instance
	 */
	public void addActionListener(UnsavedDataPanelListener listener){
		this.listeners.add(listener);
	}

	public void removeActionListener(UnsavedDataPanelListener listener){
		this.listeners.remove(listener);
	}

	public class UnsavedDataPanelListener {
		public void cancel(UnsavedDataPanel panel){

		}

		public void accept(UnsavedDataPanel panel){

		}

		public void discard(UnsavedDataPanel panel) {

		}
	}

	private class myList extends JPanel{

		/**
		 *
		 */
		private static final long serialVersionUID = 3179254463477354501L;

		private GridBagConstraints constraints;
		private GridBagConstraints constraintsLast;

		public myList(){
			super();
			initialize();
		}

		private void initialize(){
			this.setLayout(new GridBagLayout());
			this.constraints = new GridBagConstraints();
			this.constraints.fill = GridBagConstraints.HORIZONTAL;
			this.constraints.gridwidth = GridBagConstraints.REMAINDER;
			this.constraints.weightx =0;
			this.constraints.gridx =0;
			this.constraints.gridy=0;
			this.constraints.anchor = GridBagConstraints.WEST;
			this.constraints.insets =new Insets(3,3,0,0);

			this.constraintsLast = new GridBagConstraints();
			this.constraintsLast.fill = GridBagConstraints.BOTH;
			this.constraintsLast.gridheight = GridBagConstraints.REMAINDER;
			this.constraintsLast.gridwidth = GridBagConstraints.REMAINDER;
			this.constraintsLast.gridx =0;
			this.constraintsLast.weightx =1;
			this.constraintsLast.weighty =1;
			this.constraintsLast.anchor = GridBagConstraints.FIRST_LINE_START;


		}

		public void addItems(IUnsavedData[] itemList){
			int i;
			this.removeAll();
			if (itemList.length == 0){
				return;
			}

			for (i=0;i< itemList.length;i++){
				this.constraints.gridy = i;
				this.add(newItem(itemList[i]), this.constraints);
			}
			this.constraintsLast.gridy=itemList.length;
			this.add(new JLabel(),this.constraintsLast);

		}
		public int getItemsCount(){
			return this.getComponentCount() -1;
		}

		public myUnsavedItem getItem(int i){
			if (i>= this.getComponentCount()){
				return null;
			}
			return (myUnsavedItem)this.getComponent(i);
		}

		private myUnsavedItem newItem(IUnsavedData itemData){
			myUnsavedItem item;
			item = new myUnsavedItem(itemData);
			item.addMouseListener(new MouseListener(){
				public void mouseClicked(MouseEvent e) {
					myUnsavedItem item = (myUnsavedItem)e.getComponent();
					getLblResourceDescription().setText(item.getDescription());
				}
				public void mouseEntered(MouseEvent e) {
				}

				public void mouseExited(MouseEvent e) {
				}

				public void mousePressed(MouseEvent e) {
				}

				public void mouseReleased(MouseEvent e) {
				}

			});
			return item;
		}

	}

	private class myUnsavedItem extends JPanel{
		/**
		 *
		 */
		private static final long serialVersionUID = -6280057775368437349L;
		private JCheckBox chkChecked;
		private JLabel lblText;
		private IUnsavedData dataItem;

		public myUnsavedItem(IUnsavedData dataItem ){
			super();
			this.initialize();
			this.setData(dataItem);
		}

		private void initialize(){
			this.setOpaque(false);
			BorderLayout layout = new BorderLayout();
			layout.setHgap(5);
			this.setLayout(layout);
			this.add(this.getChkChecked(),BorderLayout.WEST);
			this.add(this.getLblText(),BorderLayout.CENTER);
		}

		private JLabel getLblText() {
			if (lblText == null){
				lblText = new JLabel();
				lblText.setName("lblText");
			}
			return lblText;
		}

		private JCheckBox getChkChecked() {
			if (chkChecked == null){
				chkChecked = new JCheckBox();
				chkChecked.setName("chkChecked");
				chkChecked.setSelected(true);
				chkChecked.setOpaque(false);
			}
			return chkChecked;
		}

		public boolean isSelected(){
			return getChkChecked().isSelected();
		}

		public void setSelected(boolean selected){
			getChkChecked().setSelected(selected);
		}

		public void setData(IUnsavedData dataItem){
			this.dataItem = dataItem;
			JLabel lbl = getLblText();
			if (dataItem.getIcon() != null){
			    ImageIcon ii = IconThemeHelper.getImageIcon(dataItem.getIcon());
				lbl.setIcon(ii);
			} else{
				lbl.setIcon(getBlanckIcon());
			}
			lbl.setText(dataItem.getResourceName());
			getChkChecked().setSelected(true);
		}

		public IUnsavedData getData(){
			return this.dataItem;
		}

		public String getDescription(){
			return this.dataItem.getDescription();
		}

		private ImageIcon getBlanckIcon(){
			if (blanckIcon== null){
				BufferedImage buff =new BufferedImage(16, 16, BufferedImage.TYPE_INT_ARGB);
				blanckIcon = new ImageIcon(buff);
			}
			return blanckIcon;
		}

	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == "Cancel"){
			fireDontExitEvent();
			return;
		}
		if (e.getActionCommand() == "Accept"){
			fireAcceptEvent();
			return;
		}
		if (e.getActionCommand() == "Discard") {
			fireDiscardEvent();
			return;
		}
		if (e.getActionCommand() == "SelectAll"){
			selectAll();
			return;
		}
		if (e.getActionCommand() == "DeselectAll"){
			deselectAll();
			return;
		}
	}

	public void close(){
		PluginServices.getMDIManager().closeWindow(this);

	}

	private void fireAcceptEvent() {
		Iterator iter = this.listeners.iterator();
		while (iter.hasNext()){
			((UnsavedDataPanelListener)iter.next()).accept(this);
		}
	}

	private void fireDiscardEvent() {
		Iterator iter = this.listeners.iterator();
		while (iter.hasNext()){
			((UnsavedDataPanelListener)iter.next()).discard(this);
		}
	}

	private void fireDontExitEvent() {
		Iterator iter = this.listeners.iterator();
		while (iter.hasNext()){
			((UnsavedDataPanelListener)iter.next()).cancel(this);
		}
	}

	private void setSelectedsAll( boolean selected){
		int i;
		myList theList = getList();
		myUnsavedItem item;
		for (i=0;i < theList.getItemsCount();i++){
			item = (myUnsavedItem)theList.getItem(i);
			item.setSelected(selected);
		}
		theList.doLayout();
	}

	public void deselectAll() {
		this.setSelectedsAll(false);
	}

	public void selectAll() {
		this.setSelectedsAll(true);
	}


	/**
	 * Set the resorces to ask for
	 *
	 * @param IUnsavedData[] resources pending to save
	 */
	public void setUnsavedDataArray(IUnsavedData[] unsavedDatalist){
		this.unsavedDataList = unsavedDatalist;
		myList theList = getList();

		theList.addItems(unsavedDatalist);
		theList.validate();

	}

	   /**
     * Set the resorces to ask for
     *
     * @param IUnsavedData[] resources pending to save
     */
    public void setUnsavedData(List<IUnsavedData> unsavedDatalist){
        setUnsavedDataArray(unsavedDatalist.toArray(new IUnsavedData[0]));
    }

    public List<IUnsavedData> getSelectedsUnsavedDataList(){
        return Arrays.asList(this.getSelectedsUnsavedData());
    }

	private IUnsavedData[] getUnsavedData(boolean selected){
		int i;
		myList theList = getList();
		ArrayList aList = new ArrayList();
		myUnsavedItem item;
		IUnsavedData[] result;

		for (i=0;i < theList.getItemsCount();i++){
			item = (myUnsavedItem)theList.getItem(i);
			if (item.isSelected() == selected){
				aList.add(item.getData());
			}
		}
		result = new IUnsavedData[aList.size()];
		System.arraycopy(aList.toArray(), 0, result, 0, aList.size());
		return result;
	}


	/**
	 * Returns the IUnsavedData selecteds for the user to save
	 *
	 * @return IUnsaveData[] to save
	 */
	public IUnsavedData[] getSelectedsUnsavedData(){
		return this.getUnsavedData(true);

	}

	/**
	 * Returns the IUnsavedData not selecteds for the user
	 *
	 * @return IUnsavedData[] to ignore
	 */
	public IUnsavedData[] getUnselectedsUnsavedData(){
		return this.getUnsavedData(false);
	}

	public Object getWindowProfile() {
		return WindowInfo.DIALOG_PROFILE;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"

