/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
/* CVS MESSAGES:
*
* $Id: AbstractTypeSymbolEditor.java 29596 2009-06-29 16:02:00Z jpiera $
* $Log$
* Revision 1.3  2007-08-08 11:45:38  jaume
* some bugs fixed
*
* Revision 1.1  2007/08/03 11:29:13  jaume
* refactored AbstractTypeSymbolEditorPanel class name to AbastractTypeSymbolEditor
*
* Revision 1.12  2007/08/03 09:20:47  jaume
* refactored class names
*
* Revision 1.11  2007/08/03 09:07:01  jaume
* javadoc
*
* Revision 1.10  2007/08/03 07:34:15  jaume
* javadoc
*
* Revision 1.9  2007/07/30 12:56:04  jaume
* organize imports, java 5 code downgraded to 1.4 and added PictureFillSymbol
*
* Revision 1.8  2007/07/18 06:56:03  jaume
* continuing with cartographic support
*
* Revision 1.7  2007/07/12 10:43:55  jaume
* *** empty log message ***
*
* Revision 1.6  2007/06/29 13:07:33  jaume
* +PictureLineSymbol
*
* Revision 1.5  2007/05/08 15:44:07  jaume
* *** empty log message ***
*
* Revision 1.4  2007/04/20 07:54:38  jaume
* *** empty log message ***
*
* Revision 1.3  2007/04/05 16:08:34  jaume
* Styled labeling stuff
*
* Revision 1.2  2007/03/09 11:25:00  jaume
* Advanced symbology (start committing)
*
* Revision 1.1.2.3  2007/02/21 07:35:14  jaume
* *** empty log message ***
*
* Revision 1.1.2.2  2007/02/08 15:42:13  jaume
* *** empty log message ***
*
* Revision 1.1.2.1  2007/01/26 13:49:03  jaume
* *** empty log message ***
*
* Revision 1.1  2007/01/16 11:52:11  jaume
* *** empty log message ***
*
* Revision 1.6  2006/11/13 09:15:23  jaume
* javadoc and some clean-up
*
* Revision 1.5  2006/11/06 16:06:52  jaume
* *** empty log message ***
*
* Revision 1.4  2006/10/31 16:16:34  jaume
* *** empty log message ***
*
* Revision 1.3  2006/10/30 19:30:35  jaume
* *** empty log message ***
*
* Revision 1.2  2006/10/29 23:53:49  jaume
* *** empty log message ***
*
* Revision 1.1  2006/10/27 12:41:09  jaume
* GUI
*
*
*/
package org.gvsig.app.gui.styling;

import javax.swing.JPanel;

import org.gvsig.fmap.mapcontext.rendering.symbols.CartographicSupport;
import org.gvsig.fmap.mapcontext.rendering.symbols.ISymbol;

/**
 * <p>
 * Abstract class that all the Symbol settings GUI's must extend. <b>This
 * is not Component by it self, but a bag of JPanels with the required
 * interface to be operable from the SymbolEditor</b>. The panels provide
 * by this are the tabs of the JTabbedPane of the options area in the
 * SymbolEditor.
 * </p> 
 * <p> 
 * The components contained by this panelset are automatically placed in 
 * the <b>SymbolEditor</b>'s symbol options area. In case other panel 
 * of this was already placed, then the old one is replaced by this.
 * </p>
 * <p>
 * In order to the <b>SymbolEditor</b> owner of this to decide which
 * <b>AbstractTypeSymbolEditorPanel</b> will be placed each symbol editor
 * panel must define the type of the symbol it is able to configure.
 * To do this, <b>AbstractTypeSymbolEditorPanel</b> includes the abstract
 * method <b>getSymbolClass()</b> which will return the <b>Class</b> of
 * the corresponding symbol. The symbol class must define an implementation
 * of ISymbol in general, and one of IMarkerSymbol, ILineSymbol or
 * IFillSymbol in particular, depending of the context of the symbology
 * being edited.
 * </p>
 * @author jaume dominguez faus - jaume.dominguez@iver.es
 */
public abstract class AbstractTypeSymbolEditor implements TypeSymbolEditor {
	protected SymbolEditor owner;
	private boolean applying = false;


	/**
	 * Creates a new instance of this type symbol editor panel associated to
	 * its SymbolEditor parent which will be notified of the changes in
	 * order to keep the symbol preview in sync with the user settings.
	 * @param owner, the SymbolEditor which created this.
	 */
	public AbstractTypeSymbolEditor(SymbolEditor owner) {
		this.owner = owner;
	}


	/**
	 * This overrided method returns the text shown at the SymbolEditorPanel
	 * select combobox.
	 */
	public final String toString() {
		return getName();
	}

	protected final void fireSymbolChangedEvent() {
		if (!applying) { // avoid unnecessary events
			applying = true;
			ISymbol sym = getLayer();

			if (sym instanceof CartographicSupport) {
				CartographicSupport cs = (CartographicSupport) sym;
				cs.setReferenceSystem(owner.getUnitsReferenceSystem());
				cs.setUnit(owner.getUnit());
			}
			
			owner.setLayerToSymbol(sym);
			owner.refresh();
			applying = false;
		}
	}



}

