/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
/* CVS MESSAGES:
 *
 * $Id: SymbolPreviewer.java 30892 2009-09-21 12:02:19Z cordinyana $
 * $Log: SymbolPreviewer.java,v $
 * Revision 1.6  2007/08/16 06:54:35  jvidal
 * javadoc updated
 *
 * Revision 1.5  2007/08/07 11:41:15  jvidal
 * javadoc
 *
 * Revision 1.4  2007/04/05 16:08:34  jaume
 * Styled labeling stuff
 *
 * Revision 1.3  2007/04/04 16:01:14  jaume
 * *** empty log message ***
 *
 * Revision 1.2  2007/03/09 11:25:00  jaume
 * Advanced symbology (start committing)
 *
 * Revision 1.1.2.3  2007/02/21 07:35:14  jaume
 * *** empty log message ***
 *
 * Revision 1.1.2.2  2007/02/08 15:43:05  jaume
 * some bug fixes in the editor and removed unnecessary imports
 *
 * Revision 1.1.2.1  2007/01/26 13:49:03  jaume
 * *** empty log message ***
 *
 * Revision 1.4  2007/01/16 11:52:11  jaume
 * *** empty log message ***
 *
 * Revision 1.8  2007/01/10 17:05:05  jaume
 * moved to FMap and gvSIG
 *
 * Revision 1.7  2006/10/30 19:30:35  jaume
 * *** empty log message ***
 *
 * Revision 1.6  2006/10/29 23:53:49  jaume
 * *** empty log message ***
 *
 * Revision 1.5  2006/10/29 21:45:12  jaume
 * centers in x the "none selected message"
 *
 * Revision 1.4  2006/10/29 21:43:34  jaume
 * *** empty log message ***
 *
 * Revision 1.3  2006/10/29 21:40:29  jaume
 * centers in x the "none selected message"
 *
 * Revision 1.2  2006/10/26 07:46:58  jaume
 * *** empty log message ***
 *
 * Revision 1.1  2006/10/25 10:50:41  jaume
 * movement of classes and gui stuff
 *
 * Revision 1.2  2006/10/24 19:54:55  jaume
 * *** empty log message ***
 *
 * Revision 1.1  2006/10/24 16:31:12  jaume
 * *** empty log message ***
 *
 *
 */
package org.gvsig.app.gui.styling;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.AffineTransform;

import javax.swing.JPanel;

import org.gvsig.fmap.mapcontext.MapContextLocator;
import org.gvsig.fmap.mapcontext.rendering.symbols.ISymbol;
import org.gvsig.fmap.mapcontext.rendering.symbols.SymbolDrawingException;
import org.gvsig.i18n.Messages;
import org.gvsig.symbology.fmap.mapcontext.rendering.symbol.marker.IMarkerSymbol;


/**
 * SymbolPreviewer creates a JPanel used for the user to watch the preview of
 * a symbol.It has an square form with a white background and the symbol
 * is inserted in the middle.
 * @author jaume dominguez faus - jaume.dominguez@iver.es
 *
 */
public class SymbolPreviewer extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8095930506630873031L;
	private int hGap = 3, vGap = 3;
	private ISymbol symbol;
	private EditorTool prevTool;
	private boolean useAllRoom = false; 
	/**
	 * constructor method
	 *
	 */
	public SymbolPreviewer() {
		super(true);
		setBackground(Color.WHITE);
	}
	
    public SymbolPreviewer(boolean all_room) {
        super(true);
        setBackground(Color.WHITE);
        useAllRoom = all_room;
    }
	
	/**
	 * Returns the symbol printed inside
	 * @return symbol
	 */
	public ISymbol getSymbol() {
		return symbol;
	}
	/**
	 * Establishes the symbol to be showed in the symbolpreviewer panel
	 * @param symbol
	 */
	public void setSymbol(ISymbol symbol) {
		this.symbol = symbol;
		repaint();
	}
	/**
	 * Draws the symbol in the middle of the pane in order to create a preview
	 * of the final one.
	 */
	public void paint(Graphics g) {
		super.paint(g);
		Graphics2D g2 = (Graphics2D) g;
		RenderingHints old = g2.getRenderingHints();
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		g2.translate(hGap, vGap);
		Rectangle r = getBounds();
        r = new Rectangle(0, 0,
            (int) (r.getWidth()-(hGap*2)), (int) (r.getHeight()-(vGap*2)));
		
		Double forcesize = this.forceSize(symbol);
		if (forcesize != null && !useAllRoom) {
            long difx = Math.round(0.5 * (r.getWidth() - forcesize.doubleValue()));
            long dify = Math.round(0.5 * (r.getHeight() - forcesize.doubleValue()));
		    
		    r = new Rectangle((int) difx, (int) dify,
	            (int) forcesize.doubleValue(),
	            (int) forcesize.doubleValue());
		}

		if (symbol != null) {
			try {
				symbol.drawInsideRectangle(g2, new AffineTransform(), r,null);
			} catch (SymbolDrawingException e) {
				if (e.getType() == SymbolDrawingException.UNSUPPORTED_SET_OF_SETTINGS) {
					try {
						MapContextLocator.getSymbolManager()
								.getWarningSymbol(
										SymbolDrawingException.STR_UNSUPPORTED_SET_OF_SETTINGS,
										"",
										SymbolDrawingException.UNSUPPORTED_SET_OF_SETTINGS)
								.drawInsideRectangle(g2, null, r, null);
					} catch (SymbolDrawingException e1) {
						// IMPOSSIBLE TO REACH THIS
					}
				} else {
					// should be unreachable code
					throw new Error(Messages.getText("symbol_shapetype_mismatch"));
				}
			}
		} else {
			String noneSelected = "["+Messages.getText("preview_not_available")+"]";
			FontMetrics fm = g2.getFontMetrics();
			int lineWidth = fm.stringWidth(noneSelected);
			float scale = (float) r.getWidth() / lineWidth;
			Font f = g2.getFont();
			float fontSize = f.getSize()*scale;
			g2.setFont(	f.deriveFont( fontSize ) );

			g2.drawString(noneSelected,	 (r.x*scale) - (hGap/2), r.height/2+vGap*scale);
		}
		g2.setRenderingHints(old);
	}

	/**
	 * Sets the EditorTool for the pane.
	 *
	 * @param l,EditorTool
	 */
	public EditorTool setEditorTool(EditorTool tool) {
		EditorTool previous = prevTool;

		MouseListener[] ml = getMouseListeners();
		for (int i = 0; i < ml.length; i++) {
			super.removeMouseListener(ml[i]);
		}
		MouseMotionListener[] mml = getMouseMotionListeners();
		for (int i = 0; i < mml.length; i++) {
			super.removeMouseMotionListener(mml[i]);
		}

		if (tool!= null) {
			super.addMouseListener(tool);
			setCursor(tool.getCursor());
			super.addMouseMotionListener(tool);
		}
		prevTool = tool;
		return previous;
	}
	
	private Double forceSize(ISymbol sym) {
	    
	    if (sym == null) {
	        return null;
	    }
	    
	    if (!(sym instanceof IMarkerSymbol)) {
	        return null;
	    }
	    
	    IMarkerSymbol msym = (IMarkerSymbol) sym;
	    if (msym.getUnit() == -1) {
	        /*
	         * Pixels
	         */
	        return new Double(msym.getSize());
	    } else {
	        return null;
	    }
	    
	}

}
