/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.gui.beans.panelGroup.loaders;

import java.util.ArrayList;

import org.gvsig.gui.beans.panelGroup.exceptions.ListCouldntLoadPanelException;
import org.gvsig.gui.beans.panelGroup.panels.AbstractPanel;
import org.gvsig.gui.beans.panelGroup.panels.IPanel;

/**
 * <p>Interface for all loaders which return an array of {@link AbstractPanel AbstractPanel}
 *  registered as an {@link ExtensionPoint ExtensionPoint}.</p>
 * 
 * @version 15/10/2007
 * @author Pablo Piqueras Bartolom (pablo.piqueras@iver.es) 
 */
public interface IPanelGroupLoader {
	/**
	 * <p>Loads and returns all <code>AbstractPanel</code> registered as an {@link ExtensionPoint ExtensionPoint}</p>
	 * 
	 * @param panels array with the panels or <code>null</code> if hasn't been registered any in that extension point, or that extension point didn't exit
	 * 
	 * @throws Exception last exception produced loading the panels
	 */
	public abstract void loadPanels(ArrayList<IPanel> panels) throws ListCouldntLoadPanelException;
}
