/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.gui.beans.wizard;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;

import jwizardcomponent.CancelAction;
import jwizardcomponent.DefaultJWizardComponents;
import jwizardcomponent.FinishAction;
import jwizardcomponent.common.SimpleButtonPanel;

import org.gvsig.gui.beans.wizard.panel.OptionPanel;
import org.gvsig.gui.beans.wizard.panel.OptionPanelContainer;

public class WizardPanelWithLogo extends JPanel {

    private static final long serialVersionUID = 7506729926181935234L;
    public final static int ACTION_PREVIOUS = 0;
    public final static int ACTION_NEXT = 1;
    public final static int ACTION_CANCEL = 2;
    public final static int ACTION_FINISH = 3;

    DefaultJWizardComponents wizardComponents;

    JPanel buttonPanel;
    JLabel statusLabel = new JLabel();

    ImageIcon logo;

    public WizardPanelWithLogo(ImageIcon logo) {
        this.logo = logo;
        wizardComponents = new DefaultJWizardComponents();
        init();
    }

    public WizardPanelWithLogo() {
        wizardComponents = new DefaultJWizardComponents();
        this.logo = null;
        init();
    }

    private void init() {

        this.setLayout(new BorderLayout());
        if (logo != null) {
            JPanel logoPanel = new JPanel();
            logoPanel.add(new JLabel(logo));
            logoPanel.setBackground(Color.WHITE);
            this.add(logoPanel, BorderLayout.WEST);
        }

        this.add(wizardComponents.getWizardPanelsContainer(),
            BorderLayout.CENTER);

        JPanel auxPanel = new JPanel(new BorderLayout());
        auxPanel.add(new JSeparator(), BorderLayout.NORTH);

        buttonPanel = new SimpleButtonPanel(wizardComponents);
        auxPanel.add(buttonPanel);
        this.add(auxPanel, BorderLayout.SOUTH);

        wizardComponents.setFinishAction(new FinishAction(wizardComponents) {

            public void performAction() {
                // dispose();
            }
        });
        wizardComponents.setCancelAction(new CancelAction(wizardComponents) {

            public void performAction() {
                // dispose();
            }
        });
    }

    public void doAction(int action) {
        switch (action) {
        case ACTION_NEXT:
            getWizardComponents().getNextButton().getActionListeners()[0]
                .actionPerformed(null);
            break;
        case ACTION_PREVIOUS:
            getWizardComponents().getBackButton().getActionListeners()[0]
                .actionPerformed(null);
            break;
        }
    }

    public DefaultJWizardComponents getWizardComponents() {
        return wizardComponents;
    }

    public void setWizardComponents(DefaultJWizardComponents aWizardComponents) {
        wizardComponents = aWizardComponents;
    }

    @Override
    public void show() {
        wizardComponents.updateComponents();
        super.setVisible(true);
    }

    public void addOptionPanel(OptionPanel optionPanel) {
        getWizardComponents().addWizardPanel(
            new OptionPanelContainer(getWizardComponents(), optionPanel));
    }

    public void setNextButtonEnabled(boolean isEnabled) {
        getWizardComponents().getNextButton().setEnabled(isEnabled);
    }

    public void setFinishButtonEnabled(boolean isVisible) {
        getWizardComponents().getFinishButton().setEnabled(isVisible);
    }

    public void setCancelButtonEnabled(boolean isVisible) {
        getWizardComponents().getCancelButton().setEnabled(isVisible);
    }

    private void setFinishAction(FinishAction finishAction) {
        getWizardComponents().setFinishAction(finishAction);
    }

    private void setCancelAction(CancelAction cancelAction) {
        getWizardComponents().setCancelAction(cancelAction);
    }

    public void setWizardListener(WizardPanel wizardPanel) {
        setFinishAction(new WizardPanelFinishAction(getWizardComponents(),
            wizardPanel));
        setCancelAction(new WizardPanelCancelAction(getWizardComponents(),
            wizardPanel));
    }

    public void setBackButtonEnabled(boolean isEnabled) {
        getWizardComponents().getBackButton().setEnabled(isEnabled);
    }
}
