/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.gui.beans.graphic;

import java.awt.BorderLayout;

import org.gvsig.gui.beans.TestUI;
import org.gvsig.gui.beans.buttonspanel.ButtonsPanel;
import org.gvsig.gui.beans.defaultbuttonspanel.DefaultButtonsPanel;
import org.gvsig.gui.beans.table.exceptions.NotInitializeException;
/**
 * Test para comprobar el funcionamiento de la grafica
 * 
 * @version 06/06/2008
 * @author BorSanZa - Borja Snchez Zamorano (borja.sanchez@iver.es)
 */
public class TestGraphic implements GraphicListener {
	private TestUI              frame               = new TestUI("TestGraphic");
	private DefaultButtonsPanel defaultButtonsPanel = null;
	private GraphicContainer    graphic             = null;

	public TestGraphic() throws NotInitializeException{
		graphic = new GraphicContainer(true);

		int nSeries = 3;

		String[] names = new String[nSeries];
		int[][] series = new int[nSeries][256];

		for(int iSerie = 0; iSerie < nSeries; iSerie++){
			names[iSerie] = "Band " + iSerie;
			for (int i = 0; i < 256; i++)
				series[iSerie][i] = i * (iSerie + 1);
		}
		
		graphic.getPGraphic().setViewType(0);
		
		graphic.getPGraphic().setNewChart(series, names);

		graphic.addValueChangedListener(this);
		defaultButtonsPanel = new DefaultButtonsPanel(ButtonsPanel.BUTTONS_CLOSE);
		defaultButtonsPanel.setLayout(new BorderLayout(5, 5));
		defaultButtonsPanel.setBorder(javax.swing.BorderFactory.createEmptyBorder(5, 5, 5, 5));
		defaultButtonsPanel.add(graphic, BorderLayout.CENTER);
		frame.getContentPane().add(defaultButtonsPanel);
		frame.setSize(640, 480);
		frame.setVisible(true);
	}

	public static void main(String[] args) {
		try {
			new TestGraphic();
		} catch (NotInitializeException ex){
			System.out.println("Tabla no inicializada");
		}
	}

	public void actionValueChanged(GraphicEvent e) {
		System.out.println(graphic.getX1() + ":" + graphic.getX2());
	}
}