/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.app.extension;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.project.ProjectManager;
import org.gvsig.app.project.documents.table.TableDocument;
import org.gvsig.app.project.documents.table.TableManager;
import org.gvsig.app.project.documents.table.gui.FeatureTableDocumentPanel;
import org.gvsig.app.project.documents.view.gui.AbstractViewPanel;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.mapcontext.layers.CancelationException;
import org.gvsig.fmap.mapcontext.layers.FLayer;
import org.gvsig.fmap.mapcontext.layers.FLayers;
import org.gvsig.fmap.mapcontext.layers.LayerCollectionEvent;
import org.gvsig.fmap.mapcontext.layers.LayerCollectionListener;
import org.gvsig.fmap.mapcontext.layers.LayerPositionEvent;
import org.gvsig.fmap.mapcontext.layers.vectorial.FLyrVect;

/**
 * Extensin que abre las tablas asociadas a las vistas.
 * 
 * @author Vicente Caballero Navarro
 */
public class ShowTable extends Extension implements LayerCollectionListener {

    /**
     * @see org.gvsig.andami.plugins.IExtension#isEnabled()
     */
    public boolean isEnabled() {
        AbstractViewPanel f =
            (AbstractViewPanel) PluginServices.getMDIManager()
                .getActiveWindow();

        if (f == null) {
            return false;
        }

        FLayer[] selected =
            f.getViewDocument().getMapContext().getLayers().getActives();

        boolean algunaTabla = false;

        for (int i = 0; i < selected.length; i++) {
            if (selected[i].isAvailable() && selected[i] instanceof FLyrVect) {
                FLyrVect co = (FLyrVect) selected[i];

                try {
                    if (co.getFeatureStore() != null) {
                        algunaTabla = true;
                    }
                    // } catch (ReadException e) {
                    // return false;
                } catch (NullPointerException e) {
                    return false;
                }
            }
        }

        return algunaTabla;
    }

    /**
     * @see org.gvsig.andami.plugins.IExtension#isVisible()
     */
    public boolean isVisible() {
        org.gvsig.andami.ui.mdiManager.IWindow f =
            PluginServices.getMDIManager().getActiveWindow();

        if (f == null) {
            return false;
        }

        return (f instanceof AbstractViewPanel);
    }

    /**
     * @see com.iver.mdiApp.plugins.IExtension#updateUI(java.lang.String)
     */
    public void execute(String s) {
		if ("layer-show-attributes-table".equalsIgnoreCase(s)) {
			AbstractViewPanel vista = (AbstractViewPanel) PluginServices
					.getMDIManager().getActiveWindow();
			FLayer[] actives = vista.getViewDocument().getMapContext()
					.getLayers().getActives();

			for (int i = 0; i < actives.length; i++) {
				if (actives[i] instanceof FLyrVect) {
					FLyrVect co = (FLyrVect) actives[i];

					ProjectManager projectManager = getProjectManager();
					TableManager tableManager = getTableManager();

					TableDocument projectTable = tableManager
							.getTableDocument(co);
					FeatureStore fs = ((FLyrVect) actives[i]).getFeatureStore();

					if (projectTable == null) {
						projectTable = (TableDocument) projectManager
								.createDocument(
										TableManager.TYPENAME,
										PluginServices.getText(this,
												"Tabla_de_Atributos")
												+ ": "
												+ actives[i].getName());
						projectTable.setStore(fs);
						projectTable.setAssociatedLayer(co);
						co.getParentLayer().addLayerCollectionListener(this);
						projectManager.getCurrentProject().add(projectTable);
					}

					FeatureTableDocumentPanel featureTableDocumentPanel = (FeatureTableDocumentPanel) projectTable
							.getFactory().getMainWindow(projectTable);

					featureTableDocumentPanel.getModel().setModified(true);
					PluginServices.getMDIManager().addWindow(
							featureTableDocumentPanel);
				}
			}
		}
	}

    private ProjectManager getProjectManager() {
        return ProjectManager.getInstance();
    }

    private TableManager getTableManager() {
        TableManager tableManager =
            (TableManager) getProjectManager().getDocumentManager(
                TableManager.TYPENAME);
        return tableManager;
    }

    /**
     * @see org.gvsig.andami.plugins.IExtension#initialize()
     */
    public void initialize() {
    	IconThemeHelper.registerIcon("action", "layer-show-attributes-table", this);
    }

    public void layerAdded(LayerCollectionEvent e) {
        // Nothing to do
    }

    public void layerMoved(LayerPositionEvent e) {
        // Nothing to do
    }

    public void layerRemoved(LayerCollectionEvent e) {
        FLayer layer = e.getAffectedLayer();
        // Just in case we where listening to a group of layers being removed
        // remove us from there
        if (layer instanceof FLayers) {
            ((FLayers) layer).removeLayerCollectionListener(this);
        }
        // Remove the related table document, if any
        if (layer instanceof FLyrVect) {
            getTableManager().removeTableDocument((FLyrVect) layer);
            // If the parent layers has not other child layers, for sure there
            // are not related tables opened, don't need to listen
            // LayerCollectionEvents anymore.
            // TODO: remove us also when there are not any child layers with
            // related table documents
            FLayers layers = layer.getParentLayer();
            if (layers != null && layers.getLayersCount() == 0) {
                layers.removeLayerCollectionListener(this);
            }
        }
    }

    public void layerAdding(LayerCollectionEvent e) throws CancelationException {
        // Nothing to do
    }

    public void layerMoving(LayerPositionEvent e) throws CancelationException {
        // Nothing to do
    }

    public void layerRemoving(LayerCollectionEvent e)
        throws CancelationException {
        // Nothing to do
    }

    public void visibilityChanged(LayerCollectionEvent e)
        throws CancelationException {
        // Nothing to do
    }
}
