/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.geodb.vectorialdb.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.PluginServices;
import org.gvsig.fmap.dal.store.db.DBStoreParameters;
import org.gvsig.fmap.mapcontrol.swing.dynobject.DynObjectEditor;
import org.gvsig.i18n.Messages;
import org.gvsig.tools.service.ServiceException;



/**
 * Utility class that holds a single table settings controls.
 *
 * @author jldominguez
 *
 */
public class UserTableSettingsPanel extends JPanel implements ActionListener,
    KeyListener {

    private static final long serialVersionUID = -9205704982299200912L;

    private static Logger LOG = LoggerFactory
			.getLogger(UserTableSettingsPanel.class);
    
    private FieldComboItem[] ids;
    private String initLayerName = "";
    private JComboBox idComboBox = null;
    private JTextArea sqlTextArea = null;
    private JLabel idLabel = null;
    private JLabel sqlLabel = null;
    private JCheckBox activateSQLCheckBox = null;
    protected JLabel tableNamejLabel = null;
    private JTextField layerNameTextField = null;
    private JScrollPane sqlTextAreaScroll = null;
    protected WizardDB parent = null;
	private JButton advancedSettingsButton;
	protected DBStoreParameters parameters;

    protected UserTableSettingsPanel() {
		super();
	}

	protected void setInitValues(FieldComboItem[] idComboItems,
			String initialLayerName, boolean empty, WizardDB _p,DBStoreParameters parameters) {
        parent = _p;

        initLayerName = initialLayerName;
		ids = idComboItems;
		this.parameters = parameters;

	}

    public UserTableSettingsPanel(FieldComboItem[] idComboItems,
        String initialLayerName, boolean empty, WizardDB _p,
			DBStoreParameters parameters) {
		setInitValues(idComboItems, initialLayerName, empty, _p, parameters);
        initialize(empty);
    }

    public boolean hasValidValues() {
        if (!combosHaveItems()) {
            return false;
        }

        if ((activateSQLCheckBox.isSelected()) &&
                (getSqlTextArea().getText().trim().length() == 0)) {
            return false;
        }

        if (getLayerNameTextField().getText().trim().length() == 0) {
            return false;
        }

        return true;
    }

    protected void initialize(boolean _empty) {
        tableNamejLabel = new JLabel();
        tableNamejLabel.setText(PluginServices.getText(this, "_Name"));
        tableNamejLabel.setSize(new java.awt.Dimension(100, 21));
        tableNamejLabel.setLocation(new java.awt.Point(8, 25));
        sqlLabel = new JLabel();
        // sqlLabel.setBounds(new java.awt.Rectangle(30, 90, 116, 21));
		sqlLabel.setBounds(new java.awt.Rectangle(30, 55, 150, 21));
        sqlLabel.setText(PluginServices.getText(this, "_SQL_restriction"));
        idLabel = new JLabel();
        // idLabel.setBounds(new java.awt.Rectangle(5, 55, 86, 21));
		idLabel.setBounds(new java.awt.Rectangle(260, 25, 86, 21));
        idLabel.setText(PluginServices.getText(this, "_Id_field"));

        setLayout(null);
        setBounds(new java.awt.Rectangle(3, 225, 501, 246));
        setBorder(javax.swing.BorderFactory.createTitledBorder(null,
                PluginServices.getText(this, "_Table_settings"),
                javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION,
                javax.swing.border.TitledBorder.DEFAULT_POSITION, null, null));
        add(getIdComboBox(), null);
        add(getSqlTextAreaScroll(), null);
        add(idLabel, null);
        add(sqlLabel, null);
        add(getActivateSQLCheckBox(), null);
        add(getLayerNameTextField(), null);
        add(tableNamejLabel, null);
        add(getAdvancedButton(), null);

        loadValues(_empty);
    }

    public void enableAlphaControls(boolean enable) {
        getIdComboBox().setEnabled(enable);
        getLayerNameTextField().setEnabled(enable);
        getSqlTextArea().setEnabled(enable);
        getActivateSQLCheckBox().setEnabled(enable);
        getAdvancedButton().setEnabled(enable);
    }

    public void enableSpatialControls(boolean enable) {

    }

    public void loadValues() {
        loadValues(false);
    }

    protected void loadValues(boolean is_empty) {
        if (is_empty) {
            enableAlphaControls(false);
            enableSpatialControls(false);
            getLayerNameTextField().setText("");
            getActivateSQLCheckBox().setSelected(false);
        }
        else {
            enableAlphaControls(true);
			enableSpatialControls(true);
			getActivateSQLCheckBox().setEnabled(true);
            getIdComboBox().removeAllItems();

            for (int i = 0; i < ids.length; i++) {
                getIdComboBox().addItem(ids[i]);
            }


            getLayerNameTextField().setText(initLayerName);

        }
    }

    private JComboBox getIdComboBox() {
        if (idComboBox == null) {
            idComboBox = new JComboBox();

            // idComboBox.setBounds(new java.awt.Rectangle(105, 55, 126, 21));
			idComboBox.setBounds(new java.awt.Rectangle(351, 25, 134, 21));
        }

        return idComboBox;
    }

    private JButton getAdvancedButton() {
    	if (advancedSettingsButton == null) {
    	    advancedSettingsButton = new org.gvsig.gui.beans.swing.JButton();
    	    int w = 31;
    	    advancedSettingsButton.setBounds(8, 126-w, w, w);
    	    
    	    ImageIcon ii = IconThemeHelper.getImageIcon("geodb-advanced-properties");
    	    // ImageIcon ii = IconThemeHelper.getImageIcon("geodb-connection-add");
    	    
    	    advancedSettingsButton.setIcon(ii);
       		// geodb-get-view-extent
    	    advancedSettingsButton.setToolTipText(
    	        Messages.getText("_Advanced_properties"));
    	    advancedSettingsButton.addActionListener(this);
		}
		return advancedSettingsButton;
	}

    private JCheckBox getActivateSQLCheckBox() {
        if (activateSQLCheckBox == null) {
            activateSQLCheckBox = new JCheckBox();
            activateSQLCheckBox.addActionListener(this);
            //activateSQLCheckBox
					//.setBounds(new java.awt.Rectangle(5, 90, 21, 21));
			activateSQLCheckBox
					.setBounds(new java.awt.Rectangle(5, 55, 21, 21));
        }

        return activateSQLCheckBox;
    }

    private JTextField getLayerNameTextField() {
        if (layerNameTextField == null) {
            layerNameTextField = new JTextField();
            layerNameTextField.setSize(new java.awt.Dimension(118, 21));
			// layerNameTextField.setLocation(new java.awt.Point(105, 25));
			layerNameTextField.setLocation(new java.awt.Point(120, 25));
            layerNameTextField.addKeyListener(this);
        }

        return layerNameTextField;
    }

    private JTextArea getSqlTextArea() {
        if (sqlTextArea == null) {
            sqlTextArea = new JTextArea();
            sqlTextArea.setLineWrap(true);
            sqlTextArea.setWrapStyleWord(true);
            sqlTextArea.addKeyListener(this);

            // sqlTextArea.setBounds(new java.awt.Rectangle(160,90,326,41));
        }

        return sqlTextArea;
    }

    private JScrollPane getSqlTextAreaScroll() {
        if (sqlTextAreaScroll == null) {
            sqlTextAreaScroll = new JScrollPane();
            // sqlTextAreaScroll.setBounds(new java.awt.Rectangle(160, 90, 326,
			// 41));
			sqlTextAreaScroll
					.setBounds(new java.awt.Rectangle(150, 55, 335, 53));
            sqlTextAreaScroll.setViewportView(getSqlTextArea());
            sqlTextAreaScroll.updateUI();
        }

        return sqlTextAreaScroll;
    }

    public void actionPerformed(ActionEvent e) {
        Object src = e.getSource();

        if (src == activateSQLCheckBox) {
            enableSQLSettings(activateSQLCheckBox.isSelected());
            parent.checkFinishable();
        } else if (src == getAdvancedButton()) {
        	List<String> toHide = new ArrayList<String>();
			toHide.add(DBStoreParameters.PASSWORD_PARAMTER_NAME);
			toHide.add(DBStoreParameters.HOST_PARAMTER_NAME);
			toHide.add(DBStoreParameters.PORT_PARAMTER_NAME);
			toHide.add(DBStoreParameters.USER_PARAMTER_NAME);
			toHide.add(DBStoreParameters.DBNAME_PARAMTER_NAME);
			toHide.add(DBStoreParameters.DEFAULTGEOMETRY_PARAMTER_NAME);
			toHide.add(DBStoreParameters.PKFIELDS_PARAMTER_NAME);
			toHide.add(DBStoreParameters.FIELDS_PARAMTER_NAME);
			toHide.add(DBStoreParameters.BASEFILTER_PARAMTER_NAME);
			toHide.add(DBStoreParameters.WORKINGAREA_PARAMTER_NAME);
			toHide.add(DBStoreParameters.TABLE_PARAMTER_NAME);
			toHide.add(DBStoreParameters.CRS_PARAMTER_NAME);
			try {
				parameters.validate();
			} catch (Exception ex) {
				// ignore... only for fill default values
			}
			try {
                DynObjectEditor editor = new DynObjectEditor(parameters);
                editor.editObject(true);
                } catch (ServiceException ex) {
                    LOG.error(
                        "Error creating a Swing component for the DynObject: "
                            + parameters, ex);
            }


			return;

        }

    }


    private void enableSQLSettings(boolean b) {
        sqlTextArea.setEnabled(b);
    }

    protected String getFormattedDouble(double d) {
        DecimalFormat df = new DecimalFormat("#.###");
        DecimalFormatSymbols dfs = new DecimalFormatSymbols();
        dfs.setDecimalSeparator('.');
        df.setDecimalFormatSymbols(dfs);
        return df.format(d);
    }

    public void keyPressed(KeyEvent e) {
    }

    public void keyReleased(KeyEvent e) {
        parent.checkFinishable();
    }

    public void keyTyped(KeyEvent e) {
    }


    public String getUserLayerName() {
        return getLayerNameTextField().getText();
    }

    public boolean combosHaveItems() {
        if (getIdComboBox().getItemCount() == 0) {
            return false;
        }

        return true;
    }

    public void repaint() {
        super.repaint();
        getIdComboBox().updateUI();
    }

    public String getIdFieldName() {
        return getIdComboBox().getSelectedItem().toString();
    }

    public String getWhereClause() {
        return getSqlTextArea().getText();
    }

    public boolean isSqlActive() {
        return getActivateSQLCheckBox().isSelected();
    }
}
