/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2005 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package org.gvsig.labeling.gui.layerproperties;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import org.gvsig.andami.ui.mdiManager.WindowInfo;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.fmap.geom.Geometry.TYPES;
import org.gvsig.fmap.geom.GeometryException;
import org.gvsig.fmap.mapcontext.rendering.legend.styling.IPlacementConstraints;
import org.gvsig.i18n.Messages;
import org.gvsig.labeling.placements.AbstractPlacementConstraints;
import org.gvsig.labeling.placements.LinePlacementConstraints;
import org.gvsig.labeling.placements.MultiShapePlacementConstraints;
import org.gvsig.labeling.placements.PlacementManager;
import org.gvsig.labeling.placements.PointPlacementConstraints;
import org.gvsig.labeling.placements.PolygonPlacementConstraints;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class MultiShapePlacementProperties extends JPanel
implements IPlacementProperties {
	
	private static Logger logger = LoggerFactory.getLogger(
			MultiShapePlacementProperties.class);
	private static final long serialVersionUID = 2935114466845029008L;
	private PlacementProperties pointProperties;
	private PlacementProperties lineProperties;
	private PlacementProperties polygonProperties;
	private DuplicateLayersMode dupMode;
	
	
	public MultiShapePlacementProperties(
			MultiShapePlacementConstraints constraints) throws Exception {
		
		MultiShapePlacementConstraints c = null;
		if (constraints != null) {
			c = (MultiShapePlacementConstraints) constraints.clone();
		} else {
			c = (MultiShapePlacementConstraints)
					PlacementManager.createPlacementConstraints(TYPES.GEOMETRY);
		}
		
		this.pointProperties = new PlacementProperties(
				c.getPointConstraints(), TYPES.POINT, getDuplicatesMode());
		this.lineProperties = new PlacementProperties(
				c.getLineConstraints(), TYPES.CURVE, getDuplicatesMode());
		this.polygonProperties = new PlacementProperties(
				c.getPolygonConstraints(), TYPES.SURFACE, getDuplicatesMode());
		initialize();
	}
	
	private DuplicateLayersMode getDuplicatesMode() {
		if (dupMode == null) {
			dupMode = new DuplicateLayersMode(null);
		}
		return dupMode;
	}
	
	private void initialize() { 
		setLayout(new BorderLayout());
		JPanel aux = new JPanel(new BorderLayout());
		JTabbedPane p = new JTabbedPane();
		
		p.addTab(Messages.getText("points"), pointProperties);
		p.addTab(Messages.getText("lines"), lineProperties);
		p.addTab(Messages.getText("polygon"), polygonProperties);
		setLayout(new BorderLayout());
		aux.add(p, BorderLayout.CENTER);
		aux.add(getDuplicatesMode(), BorderLayout.SOUTH);
		add(aux, BorderLayout.CENTER);
	}

	public IPlacementConstraints getPlacementConstraints() {
		return new MultiShapePlacementConstraints(
				(PointPlacementConstraints) pointProperties.getPlacementConstraints(),
				(LinePlacementConstraints) lineProperties.getPlacementConstraints(),
				(PolygonPlacementConstraints) polygonProperties.getPlacementConstraints());
				
	}
	
	public WindowInfo getWindowInfo() {
		return pointProperties.getWindowInfo();
	}
	
	public Object getWindowProfile() {
		return pointProperties.getWindowProfile();
	}
	
	
	public void actionPerformed(ActionEvent e) {
		boolean okPressed = "OK".equals(e.getActionCommand()); 
		boolean cancelPressed = "CANCEL".equals(e.getActionCommand());
		if (okPressed || cancelPressed) {
			if (okPressed) {
				try {
					pointProperties.applyConstraints();
					lineProperties.applyConstraints();
					polygonProperties.applyConstraints();
				} catch (GeometryException e1) {
					logger.error("While applying constraints.", e1);
				}
			}

			if ("CANCEL".equals(e.getActionCommand())) {
				pointProperties.constraints   = pointProperties.oldConstraints;
				lineProperties.constraints    = lineProperties.oldConstraints;
				polygonProperties.constraints = polygonProperties.oldConstraints;
			}
			
			ApplicationLocator.getManager().getUIManager().closeWindow(this);
			
			return;
		}
	}
}
