/**
 * gvSIG. Desktop Geographic Information System.
 *
 * Copyright (C) 2007-2013 gvSIG Association.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 * For any additional information, do not hesitate to contact us
 * at info AT gvsig.com, or visit our website www.gvsig.com.
 */
package org.gvsig.editing;

import java.util.List;

import javax.swing.JOptionPane;

import org.gvsig.andami.IconThemeHelper;
import org.gvsig.andami.PluginServices;
import org.gvsig.andami.plugins.Extension;
import org.gvsig.app.ApplicationLocator;
import org.gvsig.app.project.documents.view.gui.DefaultViewPanel;
import org.gvsig.editing.gui.cad.tools.ExploitCADTool;
import org.gvsig.editing.layers.VectorialLayerEdited;
import org.gvsig.fmap.dal.DALLocator;
import org.gvsig.fmap.dal.DataManager;
import org.gvsig.fmap.dal.exception.DataException;
import org.gvsig.fmap.dal.feature.FeatureSelection;
import org.gvsig.fmap.dal.feature.FeatureStore;
import org.gvsig.fmap.dal.feature.FeatureStoreProviderFactory;
import org.gvsig.fmap.geom.Geometry;
import org.gvsig.fmap.geom.type.GeometryType;
import org.gvsig.fmap.mapcontrol.MapControl;
import org.gvsig.tools.dispose.DisposableIterator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * Extensin que gestiona la simplificacin de una geometra compuesta en otras ms sencillas.
 *
 * @author Vicente Caballero Navarro
 */
public class ExploitExtension extends Extension {
    
    private static Logger logger = LoggerFactory.getLogger(ExploitExtension.class);
	private DefaultViewPanel view;

	private MapControl mapControl;
	private ExploitCADTool exploit;

	/**
	 * @see org.gvsig.andami.plugins.IExtension#initialize()
	 */
	public void initialize() {
		exploit=new ExploitCADTool();
		CADExtension.addCADTool("_exploit",exploit);

		registerIcons();
	}

	private void registerIcons(){
		IconThemeHelper.registerIcon("action", "layer-modify-explode", this);
	}

	/**
	 * @see org.gvsig.andami.plugins.IExtension#execute(java.lang.String)
	 */
	public void execute(String s) {
		CADExtension.initFocus();
		if ( "layer-modify-explode".equalsIgnoreCase(s) ) {
        	CADExtension.setCADTool("_exploit",true);
        }
		CADExtension.getEditionManager().setMapControl(mapControl);
		CADExtension.getCADToolAdapter().configureMenu();
	}

	/**
	 * @see org.gvsig.andami.plugins.IExtension#isEnabled()
	 */
	public boolean isEnabled() {
		DisposableIterator iterator = null;
		try {
			if (EditionUtilities.getEditionStatus() ==
			    EditionUtilities.EDITION_STATUS_ONE_VECTORIAL_LAYER_ACTIVE_AND_EDITABLE) {
			    
				view = (DefaultViewPanel) PluginServices.getMDIManager().getActiveWindow();
				mapControl = view.getMapControl();
				IEditionManager em = EditionLocator.getEditionManager();
				if (em.getActiveLayerEdited()==null) {
					return false;
				}
				VectorialLayerEdited vle=(VectorialLayerEdited)em.getActiveLayerEdited();
//				FLyrVect lv=(FLyrVect)vle.getLayer();
				if (((FeatureSelection)vle.getFeatureStore().getSelection()).getSize()<1) {
					return false;
				}
				
				FeatureStore store = vle.getFeatureStore();
				return canExploit(store);
				
				/*
				if (!canExploit(store)) {
				    return false;
				}
				*/
			
				
				/*
				 * We cannot iterate through selection to get enabled or disbled
				 * 
				iterator = ((FeatureSelection) vle.getFeatureStore()
						.getSelection()).fastIterator();
				if (iterator.hasNext()){
					Feature feature = (Feature) iterator.next();
					if (feature.getDefaultGeometry()!=null &&
					    exploit.isApplicable((feature.getDefaultGeometry()).getType())){
						return true;
					}
				}
				*/
			}
		} catch (Throwable e) {
		    logger.info("IsEnabled error. ", e);
		    ApplicationLocator.getManager().message("_Unable_to_update_tool_status",
		        JOptionPane.ERROR_MESSAGE);
			// NotificationManager.addError(e.getMessage(),e);
		} finally {
			if (iterator != null) {
				iterator.dispose();
			}
		}
		return false;
	}

	private boolean canExploit(FeatureStore store) {
        DataManager dalmanager = DALLocator.getDataManager();
        FeatureStoreProviderFactory factory =
            (FeatureStoreProviderFactory) dalmanager.getStoreProviderFactory(store.getProviderName());
    
        GeometryType geotype = null;
        try {
            geotype = store.getDefaultFeatureType().getDefaultGeometryAttribute().getGeomType();
        } catch (DataException e) {
            logger.info("Cant get geom type.", e);
            return false;
        }
        
        
        if (geotype.getType() == Geometry.TYPES.CURVE
            || geotype.getType() == Geometry.TYPES.MULTICURVE) {
            // exploting a curve gives curves, so ok
            return true;
        }

        if (geotype.getType() == Geometry.TYPES.SURFACE
            || geotype.getType() == Geometry.TYPES.MULTISURFACE) {
            
            // exploting a surface can give curves:
            return allowsCurvesOfSubtype(factory, geotype.getSubType());
                // (factory.allowMultipleGeometryTypes()
                // == FeatureStoreProviderFactory.YES); 
        }

        if (geotype.isTypeOf(Geometry.TYPES.GEOMETRY)) {
            // if everything is allowed, ok
            return true;
        }
        
        // otherwise: (multi)points, etc, not allowed
        return false;
	}
	
	/**
     * @param factory
     * @param subType
     * @return whether given factory supports
     * curves of given subtype
     */
    private boolean allowsCurvesOfSubtype(
        FeatureStoreProviderFactory fact,
        int subt) {
        
        List arr2_list = fact.getSupportedGeometryTypesSubtypes();
        
        if (arr2_list == null) {
            /*
             * null is a valid value, it means all supported
             */
            return true;
        }
        int[] item = null;
        int sz = arr2_list.size();
        for (int i=0; i<sz; i++) {
            item = (int[]) arr2_list.get(i);
            if (item != null && item.length == 2
                && item[0] == Geometry.TYPES.CURVE
                && item[1] == subt) {
                // found curves of same subtype (dimensions)
                return true;
            }
        }
        return false;
    }

    /**
	 * @see org.gvsig.andami.plugins.IExtension#isVisible()
	 */
	public boolean isVisible() {
		if (EditionUtilities.getEditionStatus() == EditionUtilities.EDITION_STATUS_ONE_VECTORIAL_LAYER_ACTIVE_AND_EDITABLE) {
			return true;
		}
		return false;
	}
}
